C*I4BITS -- Extract a bit field from a four byte integer.
C+
      INTEGER FUNCTION I4BITS (NUMBER, BITS, OFFSET)
      INTEGER BITS, NUMBER, OFFSET
C
C This function will extract a specified bit field from the given four
C byte integer.
C
C Returns:
C  I4BITS          : The (positive) value contained in the specified bit
C                    field.  A negative value is returned if the input
C                    parameters are incorrect.
C Arguments:
C  NUMBER (input)  : The integer containing the bit field of interest.
C  BITS   (input)  : The number of bits in the field (1 - 31).
C  OFFSET (input)  : The offset (0 relative) from the least significant
C                    bit (0 - 31).
C
C This function uses the following nonstandard intrinsic functions
C described in the software metadata files:
C
C       IAND (intger, mask)
C       ISHFT (integer, left-shift)
C
C NOTE: In order for the result of this routine to be meaningful, the
C       following condition must hold:
C
C       (BITS + OFFSET) < 33
C--
C Version 1.0  13-Feb-1990  Scott C. Allendorf
C                           - Original version.
C Version 1.1  18-May-1990  Scott C. Allendorf
C                           - Clean up error checking.
C-----------------------------------------------------------------------
C
C Assume failure.
C
      I4BITS = -1

C Check the input values for validity.

      IF ((BITS + OFFSET) .GT. 32) RETURN
      IF ((BITS .LT. 1) .OR. (BITS .GT. 31)) RETURN
      IF ((OFFSET .LT. 0) .OR. (OFFSET .GT. 31)) RETURN

C Extract the desired bit field.  If BITS is 31, we must avoid an
C arithmetic underflow while calculating the mask.

      IF (BITS .EQ. 31) THEN
         I4BITS = IAND (ISHFT (NUMBER, -OFFSET), 2147483647)
      ELSE
         I4BITS = IAND (ISHFT (NUMBER, -OFFSET), ISHFT (1, BITS) - 1)
      END IF

C Return to the calling routine

      RETURN
      END


C
C
C*I4Flip -- Flip the bytes of a four byte integer.
C+
        Integer Function I4Flip (Record)

        Integer         Record

C This function will look at the passed data record and see if the byte
C order needs to be changed on this machine.  If it does, it will swap
C the order of the bytes and check the record again to see if the byte
C reordering was successful.
C
C Returns:
C  I4Flip          : A status return value.  A value of 1 indicates
C                    successful completion, a value of 0 indicates
C                    an error.
C Arguments:
C  Record (in/out) : An Integer*4 variable
C
C This function uses the following nonstandard intrinsic functions
C described in the software metadata files:
C
C       IAND (intger, mask)
C       IOR (integer1, integer2)
C       ISHFT (integer, left-shift)
C--
C Version 1.0  25-Jan-1990  Scott C. Allendorf
C                           - Original version.
C Version 1.1  18-May-1990  Scott C. Allendorf
C                           - Avoid redundant checks.
C-----------------------------------------------------------------------
        Integer     Temp

C Assume a successful completion.

        I4Flip = 1

C Use the magic cookie in the first word to determine if a byte swap is
C necessary.

        If ((Record .ne. 99) .and. (Record .ne. 25443)) Then
            TEMP = Record       ! Save the word into temporary storage
                                ! Rewrite the word with the swapped version
            Record = Iand (Ishft (TEMP, -24), 255)
            Record = Ior (Record, Iand (Ishft (TEMP, -8), 65280))
            Record = Ior (Record, Ishft (Iand (TEMP, 65280), 8))
            Record = Ior (Record, Ishft (Iand (TEMP, 255), 24))

C Check to see if changing the byte order works on this machine by
C looking at the magic cookie.  If it is still scrambled, return an
C error condition.

            If ((Record .ne. 99) .and. (Record .ne. 25443)) I4Flip = 0
        EndIf

        Return
        End
C

C*MAF_Header    Subroutine to flip all the bytes of a MAF header to Sun order
C+
        Subroutine MAF_Header ()

C-

        Implicit        None

        Common /MAFRec/ Header, Scan
        Byte            Header (404), Scan (174)    !  174 for  35 degree MAFs,
                                                    !  600 for 120 degree MAFs,
                                                    ! 1600 for 360 degree MAFs.
        Integer*2       Iwords, Iblock, Ibytes, NCFlag, IMSYNC, IDirct (7)
        Byte            RSRV (8), MAFname (8)
        Integer*4       Ntype, Year, Day, mSec, ID, FWV, FWC, FWT, MLCa, MLCz,
     &                  Lines, NPixM, MaxPix, DIN (5), DINGR (2), DGHSK (2),
     &                  MMC, ANSUB (4), NOrbit, SCGEI (3), Spin (3),
     &                  Normal (3), UIPro, Vel (3), Sun (3), Sprat,
     &                  mSecOA, NSPNP, MnSPNP, MxSPNP

        Equivalence   (Header( 1),Iwords  ),  (Header( 97),   MMC   ),
     &                (Header( 3),Iblock  ),  (Header(101), ANSUB(1)),
     &                (Header( 5),Ibytes  ),  (Header(117),NOrbit   ),
     &                (Header( 9), Ntype  ),  (Header(121), SCGEI(1)),
     &                (Header(13),  Year  ),  (Header(133),  Spin(1)),
     &                (Header(17),   Day  ),  (Header(145),Normal(1)),
     &                (Header(21),  mSec  ),  (Header(157), UIPro   ),
     &                (Header(25),    ID  ),  (Header(161),   Vel(1)),
     &                (Header(29),   FWV  ),  (Header(173),   Sun(1)),
     &                (Header(33),   FWC  ),  (Header(185), Sprat   ),
     &                (Header(37),   FWT  ),  (Header(189),mSecOA   ),
     &                (Header(41),  MLCa  ),  (Header(193), NSPNP   ),
     &                (Header(45),  MLCz  ),  (Header(197),MnSPNP   ),
     &                (Header(49), Lines  ),  (Header(201),MxSPNP   ),
     &                (Header(53), NPixM  ),  (Header(205),NCFlag   ),
     &                (Header(57),MaxPix  ),  (Header(373),  RSRV(1)),
     &                (Header(61),  DIN(1)),  (Header(381),MAFname(1)),
     &                (Header(81),DINGR(1)),  (Header(389),IMSYNC   ),
     &                (Header(89),DGHSK(1)),  (Header(391),IDirct(1))

        Integer         I4Flip
        Real            x

!       Flip word bytes for Sun SPARCstation
        Iwords = IIor (Ishft (Iwords, 8), Ishft (Iwords, -8))
        Iblock = IIor (Ishft (Iblock, 8), Ishft (Iblock, -8))
        Ibytes = IIor (Ishft (Ibytes, 8), Ishft (Ibytes, -8))
        NCFlag = IIor (Ishft (NCFlag, 8), Ishft (NCFlag, -8))
        IMSYNC = IIor (Ishft (IMSYNC, 8), Ishft (IMSYNC, -8))
        IDirCt(1) = IIor (Ishft (IDirCt(1), 8), Ishft (IDirCt(1), -8))
        IDirCt(2) = IIor (Ishft (IDirCt(2), 8), Ishft (IDirCt(2), -8))
        IDirCt(3) = IIor (Ishft (IDirCt(3), 8), Ishft (IDirCt(3), -8))
        IDirCt(4) = IIor (Ishft (IDirCt(4), 8), Ishft (IDirCt(4), -8))
        IDirCt(5) = IIor (Ishft (IDirCt(5), 8), Ishft (IDirCt(5), -8))
        IDirCt(6) = IIor (Ishft (IDirCt(6), 8), Ishft (IDirCt(6), -8))
        IDirCt(7) = IIor (Ishft (IDirCt(7), 8), Ishft (IDirCt(7), -8))

!       Flip longword bytes for Sun SPARCstation
        x = I4Flip (Ntype)
        x = I4Flip (Year)
        x = I4Flip (Day)
        x = I4Flip (mSec)
        x = I4Flip (ID)
        x = I4Flip (UIpro)
        x = I4Flip (FWV)
        x = I4Flip (FWC)
        x = I4Flip (FWT)
        x = I4Flip (MLCa)
        x = I4Flip (MLCz)
        x = I4Flip (Lines)
        x = I4Flip (nPixM)
        x = I4Flip (MaxPix)
        x = I4Flip (DIN(1))
        x = I4Flip (DIN(2))
        x = I4Flip (DIN(3))
        x = I4Flip (DIN(4))
        x = I4Flip (DIN(5))
        x = I4Flip (DINGR(1))
        x = I4Flip (DINGR(2))
        x = I4Flip (MMC)
        x = I4Flip (DGHSK(1))
        x = I4Flip (DGHSK(2))
        x = I4Flip (ANSub(1))
        x = I4Flip (ANSub(2))
        x = I4Flip (ANSub(3))
        x = I4Flip (ANSub(4))
        x = I4Flip (nOrbit)
        x = I4Flip (SCGEI(1))
        x = I4Flip (SCGEI(2))
        x = I4Flip (SCGEI(3))
        x = I4Flip (Spin(1))
        x = I4Flip (Spin(2))
        x = I4Flip (Spin(3))
        x = I4Flip (Normal(1))
        x = I4Flip (Normal(2))
        x = I4Flip (Normal(3))
        x = I4Flip (Vel(1))
        x = I4Flip (Vel(2))
        x = I4Flip (Vel(3))
        x = I4Flip (Sun(1))
        x = I4Flip (Sun(2))
        x = I4Flip (Sun(3))
        x = I4Flip (Sprat)
        x = I4Flip (mSecOA)
        x = I4Flip (nSPNP)
        x = I4Flip (MnSPNP)
        x = I4Flip (MxSPNP)

        Return
        End


C

C*      Subroutine to flip all the bytes of a MAF scanline to Sun order
C+
        Subroutine MAF_ScanLine ()

C-

        Implicit        None

        Common /MAFRec/ Header, Scan
        Byte            Header (404), Scan (174)    !  174 for  35 degree MAFs,
                                                    !  600 for 120 degree MAFs,
                                                    ! 1600 for 360 degree MAFs.
        Integer*4       lmSec
        Integer*2       Lwords, Lbytes, DCU, Offset, NDCorr (3), Order
        Byte            MLC, MLAN, AFWP, SubCom
        Byte            Pixel (150)                 ! minimum PIXEL dimension:
                                                    !  150 for  35 degree MAFs,
                                                    !  576 for 120 degree MAFs,
                                                    ! 1576 for 360 degree MAFs.

        Equivalence     (Scan( 1),Lwords),       (Scan(12),SubCom   ),
     &                  (Scan( 3),Lbytes),       (Scan(13),   DCU   ),
     &                  (Scan( 5), lmSec),       (Scan(15),Offset   ),
     &                  (Scan( 9),   MLC),       (Scan(17),NDCorr(1)),
     &                  (Scan(10),  MLAN),       (Scan(23), Order   ),
     &                  (Scan(11),  AFWP),       (Scan(25), Pixel(1))

        Integer         I4Flip
        Real            x

!       Flip word bytes for Sun SPARCstation
        Lwords = IIor (Ishft (Lwords, 8), Ishft (Lwords, -8))
        Lbytes = IIor (Ishft (Lbytes, 8), Ishft (Lbytes, -8))
        DCU = IIor (Ishft (DCU, 8), Ishft (DCU, -8))
        Offset = IIor (Ishft (Offset, 8), Ishft (Offset, -8))
        Order = IIor (Ishft (Order, 8), Ishft (Order, -8))
        NDCorr(1) = IIor (Ishft (NDCorr(1), 8), Ishft (NDCorr(1), -8))
        NDCorr(2) = IIor (Ishft (NDCorr(2), 8), Ishft (NDCorr(2), -8))
        NDCorr(3) = IIor (Ishft (NDCorr(3), 8), Ishft (NDCorr(3), -8))

!       Flip longword bytes for Sun SPARCstation
        x = I4Flip (lmSec)

        Return
        End
