	SUBROUTINE GEO2MAG(YEAR,DAY,UTMSEC,GGLAT,GGLONG,GMLAT,GMLONG,
     &	  GMLT,GOOD)
!
!  returns corrected geomagnetic coordinates and magnetic local time
!  for the input geographic position and UT; the corrected geomagnetic
!  coordinates are interpolated from the table by Georg Gustaffson,
!  Kiruna Geophysical Institute, March 1983
!
!			name	type	description
!
!  input arguments:	YEAR	INT*2	year mod 100
!			DAY	INT*2	day of year
!			UTMSEC	INT*4	milliseconds of the day
!			GGLAT	REAL*4	geographic north latitude (degrees)
!			GGLONG	REAL*4	geographic east longitude (degrees)
!
! output arguments:	GMLAT	REAL*4	corrected geomagnetic n. latitude  (deg)
!			GMLONG	REAL*4	corrected geomagnetic e. longitude (deg)
!			GMLT	REAL*4	magnetic local time (degrees)
!			GOOD	LOG*1	.TRUE. if geomagnetic coordinates 
!					were successfully calculated, 
!					otherwise .FALSE.
!
!
	REAL*4		GGLAT,GGLONG,GMLAT,GMLONG,GMLT
	INTEGER*4	UTMSEC,SOLMSEC
	INTEGER*2	YEAR,DAY
	REAL*4		GST,SLONG,SRASN,SDEC
	LOGICAL*1	GOOD
!
	CALL GEO2CGM(GGLAT,GGLONG,GMLAT,GMLONG,GOOD)
	CALL SUN(YEAR,DAY,UTMSEC,GST,SLONG,SRASN,SDEC)
	CALL EQTIME(YEAR,DAY,UTMSEC,SOLMSEC)
	CALL MLT(SOLMSEC,SDEC,GGLAT,GGLONG,GMLT)
!
	RETURN
	END
!
!
!
	SUBROUTINE GEO2CGM(GGLAT,GGLONG,GMLAT,GMLONG,GOOD)
!
!  converts geographic coordinates in degrees to corrected geomagnetic
!  coordinates in degrees by interpolation from the table of Georg Gustaffson,
!  Kiruna Geophysical Institute, March 1983
!
!			name	type	description
!
!  input arguments:	GGLAT	REAL*4	geographic north latitude
!			GGLONG	REAL*4	geographic east longitude
!
! output arguments:	GMLAT	REAL*4	corrected geomagnetic north latitude
!			GMLONG	REAL*4	corrected geomagnetic east longitude
!			GOOD	LOG*1	.TRUE. if coordinate conversion 
!					was successful,
!					.FALSE. if coordinates could not 
!					be computed for any reason
!
!
!
	COMMON /CGMCRDS/CGMC
!
	REAL*4		CGMC(72,153)
	REAL*4		CMLAT(4),CMLONG(4),CGLAT(4),CGLONG(4)
	REAL*4		GGLAT,GGLONG,GMLAT,GMLONG,GGLNG
	INTEGER*2	LAT,NUMBER,L,M,K,N
	LOGICAL*1	GOOD
	EXTERNAL	CGMCRD
!
!---compute column number and corresponding latitude
	IF (GGLAT.LT.24.0.AND.GGLAT.GT.-12.0) GO TO 90
	LAT = IFIX(GGLAT)/2 * 2
	IF (IABS(LAT).EQ.90) LAT = ISIGN(88,LAT)
	NUMBER = ( LAT+1-ISIGN(17,LAT) )/2 + 36
	GGLNG = AMOD(GGLONG,360.)
	IF (GGLNG.LT.0.) GGLNG = GGLNG + 360.
	L = IFIX(GGLNG)/10 + 1
	M = MOD(L,36) + 1
	IF (LAT.EQ.-12.AND.L.GT.26) GO TO 90
	K = 1
!
!---get closest coordinate pairs from this record
 31	CMLAT(K) = CGMC(L*2-1,NUMBER)
	CMLONG(K) = CGMC(L*2,NUMBER)
	CGLAT(K) = FLOAT(LAT)
	CGLONG(K) = FLOAT(L-1)*10.0
	CMLAT(K+1) = CGMC(M*2-1,NUMBER)
	CMLONG(K+1) = CGMC(M*2,NUMBER)
	CGLAT(K+1) = FLOAT(LAT)
	CGLONG(K+1) = FLOAT(M-1)*10.0
	IF (CGLONG(K+1).EQ.0.) CGLONG(K+1) = 360.
	IF (CMLONG(K+1).LT.CMLONG(K)-220.)
     &    CMLONG(K+1) = CMLONG(K+1) + 360.
	IF (CMLONG(K).LT.CMLONG(K+1)-220.)
     &    CMLONG(K) = CMLONG(K) + 360.
!
!---get another coordinate record if needed
	N = K
	IF (GGLAT.EQ.CGLAT(N).AND.GGLNG.EQ.CGLONG(N)) GO TO 75
	N = K + 1
	IF (GGLAT.EQ.CGLAT(N).AND.GGLNG.EQ.CGLONG(N)) GO TO 75
	IF (K.EQ.3) GO TO 66
	K = K + 2
	IF (IABS(LAT).EQ.88) GO TO 50
 40	NUMBER = NUMBER + ISIGN(1,LAT)
	LAT = LAT + ISIGN(2,LAT)
	GO TO 31
!
!---get points in lats 88. to 90. or -88. to -90.
 50	IF (LAT.LT.0) GO TO 52
	CMLAT(3) = 82.0
	CMLONG(3) = 171.0
	CGLAT(3) = 90.
	GO TO 53
 52	CMLAT(3) = -73.7
	CMLONG(3) = 17.9
	CGLAT(3) = -90.
 53	CMLAT(4) = CMLAT(3)
	CMLONG(4) = CMLONG(3)
	CGLONG(3) = CGLONG(1)
	CGLAT(4) = CGLAT(3)
	CGLONG(4) = CGLONG(2)
!
!---use closest point to get coordinates of input point
 66	IF (CMLONG(1)-CMLONG(3).LT.220.) GO TO 67
	CMLONG(3) = CMLONG(3) + 360.
	CMLONG(4) = CMLONG(4) + 360.
	GO TO 68
 67	IF (CMLONG(3)-CMLONG(1).LT.220.) GO TO 68
	CMLONG(1) = CMLONG(1) + 360.
	CMLONG(2) = CMLONG(2) + 360.
 68	CALL INTRP4(CGLAT,CGLONG,CMLAT,CMLONG,GGLAT,GGLNG,
     &	  GMLAT,GMLONG) 
	GMLONG = AMOD(GMLONG,360.)
	GOOD = .TRUE.
	GO TO 98
!
 75	GMLAT = CMLAT(N)
	GMLONG = CMLONG(N)
	GOOD = .TRUE.
	GO TO 98
!
!---error exit
 90	GOOD = .FALSE.
!
!
 98	CLOSE(UNIT=1)
 99	RETURN
	END      
!
!
!
	SUBROUTINE INTRP4(RLATS,RLONGS,SLATS,SLONGS,RLAT,RLONG,
     &    SLAT,SLONG)
!
!  interpolates among four sets of latitude and longitude pairs
!  in two coordinate systems for a fifth point
!
!			name	type	description
!
!  input arguments:	RLATS	REAL*4	latitudes in coordinate system 1
!			RLONGS	REAL*4	longitudes in system 1
!			SLATS	REAL*4	latitudes in system 2
!			SLONGS	REAL*4	longitudes in system 2
!			RLAT	REAL*4  lat in system 1 of interpolation point
!			RLONG	REAL*4	long in system 1 of interpolation point
!
! output arguments:	SLAT	REAL*4	interpolated latitude in system 2
!			SLONG	REAL*4	interpolated longitude in system 2
!
!
	REAL*4 RLATS(4),RLONGS(4),SLATS(4),SLONGS(4),
     &	    RLAT,RLONG,SLAT,SLONG,ALONG1,ALONG2,ALAT1,ALAT2
!
	ALONG1 = (RLONG-RLONGS(1))/(RLONGS(2)-RLONGS(1))
	ALONG2 = 1. - ALONG1
	ALAT1 = (RLAT-RLATS(1))/(RLATS(3)-RLATS(1))
	ALAT2 = 1. - ALAT1
!
	SLONG = ALONG2*ALAT2*SLONGS(1) + ALONG1*ALAT2*SLONGS(2)
     &	  + ALONG2*ALAT1*SLONGS(3) + ALONG1*ALAT1*SLONGS(4)
	SLAT = ALONG2*ALAT2*SLATS(1) + ALONG1*ALAT2*SLATS(2)
     &	  + ALONG2*ALAT1*SLATS(3) + ALONG1*ALAT1*SLATS(4)
!
	RETURN
	END
!
!
!
	SUBROUTINE CGM2GEO(GMLAT,GMLONG,GGLAT,GGLONG,GOOD)
!
!  converts corrected geomagnetic coordinates in degrees to geographic
!  coordinates in degrees by interpolation from a table based on that
!  of Georg Gustaffson, Kiruna Geophysical Institute, March 1983
!
!
!			name	type	description
!
!  input parameters:	GMLAT	REAL*4	corrected geomagnetic north latitude
!			GMLONG	REAL*4	corrected geomagnetic east longitude
!
! output parameters:	GGLAT	REAL*4	geographic north latitude
!			GGLONG	REAL*4	geographic east longitude
!			GOOD	LOG*1	.TRUE. if coordinate conversion 
!					was successful,
!					.FALSE. if coordinates could not 
!					be computed for any reason
!
!
	COMMON /CGMCRDS/CGMC
!
	REAL*4		CGMC(72,153)
	REAL*4		CMLAT(4),CMLONG(4),CGLAT(4),CGLONG(4)
	REAL*4		GMLAT,GMLONG,GGLAT,GGLONG,GMLNG
	INTEGER*2	LAT,NUMBER,L,M,K,N
	LOGICAL*1	GOOD
	EXTERNAL	CGMCRD
!
!---compute number and corresponding latitude
	LAT = IFIX(GMLAT)/2 * 2
	IF (LAT.EQ.0.OR.LAT.EQ.2) GO TO 90 
	IF (IABS(LAT).EQ.90) LAT = LAT - ISIGN(2,LAT)
	NUMBER = LAT/2 - ISIGN(4,LAT) + 113
	GMLNG = AMOD(GMLONG,360.)
	IF (GMLNG.LT.0.) GMLNG = GMLNG + 360.
	L = IFIX(GMLNG)/10 + 1
	M = MOD(L,36) + 1
	K = 1
!
!---get closest coordinate pairs from this record
 31	CGLAT(K) = CGMC(L*2-1,NUMBER)
	CGLONG(K) = CGMC(L*2,NUMBER)
	CMLAT(K) = FLOAT(LAT)
	CMLONG(K) = FLOAT(L-1)*10.0
	CGLAT(K+1) = CGMC(M*2-1,NUMBER)
	CGLONG(K+1) = CGMC(M*2,NUMBER)
	CMLAT(K+1) = FLOAT(LAT)
	CMLONG(K+1) = FLOAT(M-1)*10.0
	IF (CMLONG(K+1).EQ.0.) CMLONG(K+1) = 360.
	IF (CGLONG(K+1).LT.CGLONG(K)-220..AND.CGLONG(K+1).NE.-1.) 
     &	  CGLONG(K+1) = CGLONG(K+1) + 360.
	IF (CGLONG(K).LT.CGLONG(K+1)-220..AND.CGLONG(K).NE.-1.) 
     &	  CGLONG(K) = CGLONG(K) + 360.
!
!---get another coordinate record if needed
	N = K
	IF (GMLAT.EQ.CMLAT(N).AND.GMLNG.EQ.CMLONG(N)) GO TO 75
	N = K + 1
	IF (GMLAT.EQ.CMLAT(N).AND.GMLNG.EQ.CMLONG(N)) GO TO 75
	IF (K.EQ.3) GO TO 66
	K = K + 2
	IF (IABS(LAT).EQ.88) GO TO 50
 40	NUMBER = NUMBER + ISIGN(1,LAT)
	LAT = LAT + ISIGN(2,LAT)
	GO TO 31
!
!---get points in lats 88. to 90. or -88. to -90.
 50	IF (LAT.LT.0) GO TO 52
	CGLAT(3) = 80.6
	CGLONG(3) = 278.5
	CMLAT(3) = 90.
	GO TO 53
 52	CGLAT(3) = -74.0
	CGLONG(3) = 126.3
	CMLAT(3) = -90.
 53	CGLAT(4) = CGLAT(3)
	CGLONG(4) = CGLONG(3)
	CMLONG(3) = CMLONG(1)
	CMLAT(4) = CMLAT(3)
	CMLONG(4) = CMLONG(2)
!
!---use closest point to get coordinates of input point
 66	IF (CGLONG(1).EQ.-1..OR.CGLONG(2).EQ.-1.
     &   .OR.CGLONG(3).EQ.-1..OR.CGLONG(4).EQ.-1.) GO TO 90
	IF (CGLONG(1)-CGLONG(3).LT.220.) GO TO 67
	CGLONG(3) = CGLONG(3) + 360.
	CGLONG(4) = CGLONG(4) + 360.
	GO TO 68
 67	IF (CGLONG(3)-CGLONG(1).LT.220.) GO TO 68
	CGLONG(1) = CGLONG(1) + 360.
	CGLONG(2) = CGLONG(2) + 360.
 68	CALL INTRP4(CMLAT,CMLONG,CGLAT,CGLONG,GMLAT,GMLNG,
     &	  GGLAT,GGLONG) 
	GGLONG = AMOD(GGLONG,360.)
	GOOD = .TRUE.
	GO TO 98
!
 75	IF (CGLONG(N).EQ.-1.) GO TO 90
	GGLAT = CGLAT(N)
	GGLONG = CGLONG(N)
	GOOD = .TRUE.
	GO TO 98
!
!---error exit
 90	GOOD = .FALSE.
!
!
 98	CLOSE(UNIT=1)
 99	RETURN
	END
!
!
!
	SUBROUTINE MLT(SOLMSEC,SDEC,GGLAT,GGLONG,GMLT)
!
!  calculates magnetic local time for the input geographic position,
!  universal time, and sun declination
!
!			name	type	description
!
!  input arguments:	SOLMSEC	INT*4	apparent solar time in milliseconds
!			SDEC	REAL*4	declination of sun in degrees
!			GGLAT	REAL*4	geographic north latitude
!			GGLONG	REAL*4	geographic east longitude
!
!  output argument:	GMLT	REAL*4	magnetic local time in degrees
!
!
	INTEGER*4 SOLMSEC
	REAL*4 SDEC,GGLAT,GGLONG,GMLT
	REAL*4 POLEMN(2),POLEMS(2)
	REAL*4 GM1,GM2,GM3,X1,X2,X3,Y1,Y2,Y3,ZLAT,ZLONG,Z1,Z2,Z3,
     &	   XX,XY,XZ
	REAL*4 SUNPHI,SUNX,SUNY,SUNZ,ZDOTS,G1,G2,G3,R,TH
	DATA POLEMN/80.6,278.5/,POLEMS/-74.0,126.3/
!
!---set up coordinate basis for anti-solar magnetic system
!
!---let north magnetic pole or reverse of south magnetic pole be Z-axis
	IF (GGLAT.GT.0.) THEN
	    ZLAT = POLEMN(1)
	    ZLONG = POLEMN(2)
	ELSE
	    ZLAT = -POLEMS(1)
	    ZLONG = AMOD(POLEMS(2)+180.0,360.0)
	ENDIF
	CALL SPHDREC(1.,(90.0-ZLAT),ZLONG,Z1,Z2,Z3)
!
!---get geographic sun vector
	SUNPHI = (43200000-SOLMSEC)*.4166667E-5
	CALL SPHDREC(1.,90.-SDEC,SUNPHI,SUNX,SUNY,SUNZ)
!
!---let X-axis be reverse of sun vector projection on plane orthogonal
!		to Z-axis
	ZDOTS = Z1*SUNX + Z2*SUNY + Z3*SUNZ
	XX = - (SUNX - ZDOTS*Z1)
	XY = - (SUNY - ZDOTS*Z2)
	XZ = - (SUNZ - ZDOTS*Z3)
	CALL NRMLIZ(XX,XY,XZ,X1,X2,X3)
!
!---get Y-axis by Z-axis cross X-axis
	CALL CROSS(Z1,Z2,Z3,X1,X2,X3,Y1,Y2,Y3)
!
!---rotate input point to new system
	CALL SPHDREC(1.,90.-GGLAT,GGLONG,G1,G2,G3)
	GM1 = G1*X1 + G2*X2 + G3*X3
	GM2 = G1*Y1 + G2*Y2 + G3*Y3
	GM3 = G1*Z1 + G2*Z2 + G3*Z3
!
!---magnetic local time is phi
	CALL RECSPHD(GM1,GM2,GM3,R,TH,GMLT)
!
!
	RETURN
	END
!
!
!
	SUBROUTINE MAG2GEO(GMLAT,MLT,YEAR,DAY,UTMSEC,GGLAT,GGLONG,GOOD)
!
!  calculates geographic coordinates for the input corrected geomagnetic
!  latitude, magnetic local time, and UT
!
!			name	type	description
!
!  input arguments:	GMLAT	REAL*4	corrected geomagnetic latitude (deg)
!			MLT	REAL*4	magnetic local time (degrees)
!			YEAR	INT*2	year mod 100	       \
!			DAY	INT*2	day of year		>universal
!			UTMSEC	INT*4	milliseconds of the day/    time
!
! output arguments:	GGLAT	REAL*4	geographic north latitude (deg)
!			GGLONG	REAL*4	geographic east longitude (deg)
!			GOOD	LOG*1	.TRUE. if all conversions are
!					  successful, .FALSE. otherwise
!
!
	INTEGER*4 UTMSEC,SOLMSEC
	INTEGER*2 YEAR,DAY,I
	REAL*4 MLT,SDEC,GGLAT,GGLONG,GMLAT,GMLAT1,GMLAT1L,GMLAT1S,
     &	   GMLAT2,GMLAT2L,GMLNG2,GST,SLONG,SRASN,GMLD
	REAL*4 POLEMN(2),POLEMS(2)
	REAL*4 GMX,GMY,GMZ,GGX,GGY,GGZ,X1,X2,X3,Y1,Y2,Y3,XX,XY,XZ,
     &	   ZLAT,ZLONG,Z1,Z2,Z3,THETA,GTHETA,R
	REAL*4 SUNPHI,SUNX,SUNY,SUNZ,ZDOTS
	LOGICAL*1 GOOD
	DATA POLEMN/80.6,278.5/,POLEMS/-74.0,126.3/
!
	GMLAT1L = 0.
	GMLAT2L = 0.
	I = 0
!
!---set up coordinate basis for anti-solar magnetic system
!
!---let north magnetic pole or reverse of south magnetic pole be Z-axis
	IF (GMLAT.GT.0.) THEN
	    ZLAT = POLEMN(1)
	    ZLONG = POLEMN(2)
	ELSE
	    ZLAT = -POLEMS(1)
	    ZLONG = AMOD(POLEMS(2)+180.0,360.0)
	ENDIF
	CALL SPHDREC(1.,(90.-ZLAT),ZLONG,Z1,Z2,Z3)
!
!---get geographic sun vector
	CALL SUN(YEAR,DAY,UTMSEC,GST,SLONG,SRASN,SDEC)
	CALL EQTIME(YEAR,DAY,UTMSEC,SOLMSEC)
	SUNPHI = (43200000.-SOLMSEC)*.4166667E-5
	CALL SPHDREC(1.,90.-SDEC,SUNPHI,SUNX,SUNY,SUNZ)
!
!---let X-axis be reverse of sun vector projection on plane orthogonal
!		to Z-axis
	ZDOTS = Z1*SUNX + Z2*SUNY + Z3*SUNZ
	XX = - (SUNX - ZDOTS*Z1)
	XY = - (SUNY - ZDOTS*Z2)
	XZ = - (SUNZ - ZDOTS*Z3)
	CALL NRMLIZ(XX,XY,XZ,X1,X2,X3)
!
!---get Y-axis by Z-axis cross X-axis
	CALL CROSS(Z1,Z2,Z3,X1,X2,X3,Y1,Y2,Y3)
!
!---the MLT gives PHI in this system
!
!---the GMLAT gives approximately THETA
	GMLAT1 = GMLAT
	IF (ABS(GMLAT).GT.89.5) THEN
	    IF (GMLAT.LT.0.) THEN
		GGLAT = POLEMS(1)
		GGLONG = POLEMS(2)
	    ELSE
		GGLAT = POLEMN(1)
		GGLONG = POLEMN(2)
	    ENDIF
	    GO TO 65
	ENDIF
 55	THETA = 90. - GMLAT1
!
!---rotate to geographic
	CALL SPHDREC(1.,THETA,MLT,GMX,GMY,GMZ)
	GGX = GMX*X1 + GMY*Y1 + GMZ*Z1
	GGY = GMX*X2 + GMY*Y2 + GMZ*Z2
	GGZ = GMX*X3 + GMY*Y3 + GMZ*Z3
	CALL RECSPHD(GGX,GGY,GGZ,R,GTHETA,GGLONG)
	GGLAT = 90. - GTHETA
!
!---check this answer
 65	CALL GEO2CGM(GGLAT,GGLONG,GMLAT2,GMLNG2,GOOD)
	IF (.NOT.GOOD) RETURN
	GMLD = ABS(GMLAT2-GMLAT)
	IF (GMLD.LT..05) RETURN
	IF (ABS(GMLAT).GT.89.) THEN
	    IF (GMLD.LT..2) RETURN
	    IF (GMLD.LT.ABS(GMLAT)-89.) RETURN
	ENDIF
!
!---prepare for iteration
	I = I + 1
	IF (I.GT.20) THEN
	    GOOD = .FALSE.
	    RETURN
	ENDIF
	GMLAT1S = GMLAT1
	IF (GMLAT2L.EQ.0.) THEN
	    GMLAT1 = GMLAT1 + (GMLAT-GMLAT2)*SIGN(1.,90.-ABS(GMLAT1))
	ELSE
	    IF (SIGN(1.,GMLAT2-GMLAT).EQ.SIGN(1.,GMLAT2L-GMLAT)) THEN
		IF (ABS(GMLAT2-GMLAT).LE.ABS(GMLAT2L-GMLAT)) THEN
		    GMLAT1 = GMLAT1 + (GMLAT1-GMLAT1L)*(GMLAT-GMLAT2)
     &		      /(GMLAT2-GMLAT2L)
		ELSE
		    GMLAT1 = GMLAT1L + (GMLAT1-GMLAT1L)*.5
		ENDIF
	    ELSE
		GMLAT1 = (GMLAT1+GMLAT1L)/2.
	    ENDIF
	ENDIF
	GMLAT1L = GMLAT1S
	GMLAT2L = GMLAT2
	GO TO 55
!
!
	END
!
!
!
	SUBROUTINE MAGSAT(YEAR,DAY,RKM,GLAT,GLONG,BR,BT,BP,B)
!
!  calculates the magnetic field using the MAGSAT model;
!  based on the ONEMAG progam in document NSSDC 72-12 from the
!  National Space Science Data Center
!
!-------------------------------------------------------------------------! 
!  MODIFICATION WITH MAGSAT COEFFICIENTS BY D.R. WEIMER, 2-17-84 	  !
!  SECULAR VARIATION COEFFICIENTS FROM IGRF 1980 MODEL 			  !
!									  !
!  TMOLD TIME OF ONEMAG REMOVED BY WEIMER.  REPLACED BY CRAVEN, 12 OCT 84.!
!-------------------------------------------------------------------------!
!
!
!			name	type	description
!
!  input arguments:	YEAR	INT*2	year mod 100
!			DAY	INT*2	day of year
!			RKM	REAL*4	radial distance in kilometers
!			GLAT	REAL*4	geocentric latitude
!			GLONG	REAL*4	east longitude
!
! output arguments:	BR	REAL*4\
!			BT	REAL*4 >geocentric field components
!			BP	REAL*4/		in gauss
!			B	REAL*4	field magnitude in gauss
!
!
	INTEGER*2	YEAR,DAY,NMAX,M,N,JJ,YEAR4
	REAL*4		RKM,GLAT,GLONG,BR,BT,BP,B,TZERO,TM,TMOLD,T,
     &			G(11,11),SHMIT(11,11),CONST(11,11),GT(11,11),
     &			FN(11),FM(11),SP(11),CP(11),GS(11,11),
     &			ST,CT
	DOUBLE PRECISION AR,P(11,11),DP(11,11),PAR,TEMP,DBR,DBT,DBP,AOR
	DATA NMAX/11/,SHMIT(1,1)/0./,TZERO/1979.85/,TMOLD/0.0/ 
	DATA P(1,1)/1.D0/,CP(1)/1.0/,DP(1,1)/0.D0/,SP(1)/0.0/
!
	DATA G / 1.,-29989.6,-1994.8,1279.9,938.3,-217.4,48.3,71.7, 
     &   18.4,5.6,-3.3,5608.1,-1958.6,3027.2,-2179.8,782.5,357.6, 
     &   65.2,-59.,6.8,10.4,-3.5,-2127.3,-196.1,1661.6,1251.4,398.4,
     &   261.,41.4,1.6,-0.1,1.1,2.5,-334.4,270.7,-251.1,833.0,-419.2, 
     &   -73.9,-192.2,20.5,-10.8,-12.6,-5.3,211.6,-256.7,52.,-297.6,
     &   199.3,-162.,3.5,-12.6,-7.,9.5,-2.1,45.2,149.4,-150.3,-78.1,
     &   91.8,-48.3,13.7,0.6,4.3,-3.3,4.6,-14.5,93.4,70.6,-42.9,-2.4, 
     &   16.9,-107.6,10.6,2.7,-1.3,3.1,-82.4,-27.5,-4.9,16.1,18.1,
     &   -22.9,-9.9,-2.0,6.3,6.8,0.6,6.9,-17.9,4.0,-22.3,9.2,16.1,
     &   -13.1,-14.8,-1.2,1.4,1.8,-21.1,15.2,8.9,-4.8,-6.5,9.,9.5,
     &   -5.9,2.1,-5.1,2.8,1.4,0.4,2.6,5.6,-4.2,-0.4,-1.3,3.5,-0.5, 
     &   -6.2,-0.5 /
	DATA GT / 1.,22.4,-18.3,0.,-1.4,1.5,0.4,-1.0,0.8,2*0.,
     &   -15.9,11.3,3.2,-6.5,-1.4,0.4,0.0,-0.8,-0.2,2*0., 
     &   -12.7,-25.2,7.,-0.7,-8.2,-0.8,3.4,0.4,-0.3,2*0., 
     &   0.2,2.7,-7.9,1.0,-1.8,-3.3,0.8,0.5,0.,2*0.,
     &   4.6,1.6,2.9,0.4,-5.0,0.2,0.8,1.6,-0.8,2*0.,
     &   1.8,-0.4,0.,1.3,2.1,1.4,0.3,0.1,-0.2,2*0., 
     &   -0.5,-1.4,0.,-1.6,0.5,0.,-0.1,0.1,0.7,2*0.,
     &   -0.4,0.4,0.2,1.4,-0.5,-0.1,1.1,0.0,-0.3,2*0.,
     &   -0.1,-0.7,0.,-0.8,0.2,0.2,-1.1,0.8,1.2,24*0. / 
!
	IF (SHMIT(1,1).EQ.-1.) GO TO 8 
!
!---initialization on first call
	SHMIT(1,1) = -1.
	DO 2 N=1,NMAX
	FN(N) = N 
	DO 1 M=1,NMAX
	FM(M) = M - 1
	CONST(N,M) = FLOAT( (N-2)**2 - (M-1)**2 )/((2*N-3)*(2*N-5))
  1	CONTINUE
  2	CONTINUE
	DO 4 N=2,NMAX 
	SHMIT(N,1) = (2*N-3)*SHMIT(N-1,1)/(N-1)
	JJ=2 
	DO 3 M=2,N 
	SHMIT(N,M) = SHMIT(N,M-1)*SQRT(FLOAT((N-M+1)*JJ)/(N+M-2)) 
	SHMIT(M-1,N) = SHMIT(N,M) 
	JJ=1
  3	CONTINUE
  4	CONTINUE
	DO 7 N=1,NMAX 
	DO 6 M=1,NMAX 
	GT(N,M) = GT(N,M)*SHMIT(N,M)
	G(N,M) = G(N,M)*SHMIT(N,M)
  6	CONTINUE
  7	CONTINUE
!
!---adjust to time in years
  8	TM = YEAR4(YEAR) + DAY/(366.-MIN0(MOD(YEAR,4),1))
	IF (TM.EQ.TMOLD) GO TO 11
	TMOLD = TM
	T = TM - TZERO
	DO 10 N=1,NMAX
	DO 9 M=1,NMAX
	GS(N,M)=G(N,M) + T*GT(N,M)
  9	CONTINUE
 10	CONTINUE
!
!---get angle functions
 11	ST = SIND(90.-GLAT)
	CT = COSD(90.-GLAT)
!
!---calculation begins here
	SP(2) = SIND(GLONG) 
	CP(2) = COSD(GLONG)
	DO 12 M=3,NMAX
	SP(M) = SP(2)*CP(M-1) + CP(2)*SP(M-1)
 12	CP(M) = CP(2)*CP(M-1) - SP(2)*SP(M-1)
	AOR = 6371.2/RKM
	AR = AOR*AOR
	DBR = 0.D0
	DBT = 0.D0
	DBP = 0.D0
	DO 18 N=2,NMAX
	AR = AOR*AR
	DO 17 M=1,N
	IF (M.EQ.N) GO TO 13 
	P(N,M) = CT*P(N-1,M) - CONST(N,M)*P(N-2,M)
	DP(N,M) = CT*DP(N-1,M) - ST*P(N-1,M) - CONST(N,M)*DP(N-2,M) 
	GO TO 14
 13	P(N,N) = ST*P(N-1,N-1)
	DP(N,N) = ST*DP(N-1,N-1) + CT*P(N-1,N-1)
 14	PAR = P(N,M)*AR 
	IF (M.EQ.1) GO TO 15 
	TEMP = GS(N,M)*CP(M) + GS(M-1,N)*SP(M)
	DBP = DBP - (GS(N,M)*SP(M) - GS(M-1,N)*CP(M))*FM(M)*PAR 
	GO TO 16
 15	TEMP = GS(N,M) 
 16	DBR = DBR - TEMP*FN(N)*PAR
	DBT = DBT + TEMP*DP(N,M)*AR 
 17	CONTINUE
 18	CONTINUE
	IF (ST.EQ.0.) THEN
	    BP = 0.
	ELSE
	    BP = DBP/ST/100000.
	ENDIF
	BR = DBR/100000. 
	BT = DBT/100000. 
	B = SQRT( BR*BR + BT*BT + BP*BP )
!
	RETURN
	END
!
!
!
	SUBROUTINE LINE_TRACE(YEAR,DAY,GLAT,GLONG,RKM,DIR,HALT,PLAT,PLON,GOOD)
!
!
!  traces the magnetic field line from the specified point in either
!  direction to any specified altitude using the MAGSAT model; 
!  based on routine designed by E.G. Stassinopoulos and G.D. Mead,
!  code 641, GSFC, Jan. 1971
!
!
!			name	type	description
!
!  input arguments:	YEAR	INT*2	year mod 100
!			DAY	INT*2	day of year
!			GLAT	REAL*4	geocentric north latitude in degrees
!			GLONG	REAL*4	east longitude in degrees
!			RKM	REAL*4	radial distance in kilometers
!			DIR	INT*2	direction of trace,
!					  -1 for lower along field line,
!					  +1 for higher to opposite hemisphere,
!					  0 for higher to same hemisphere
!			HALT	REAL*4	geocentric altitude of intersect
!
! output arguments:	PLAT	REAL*4	geocentric north lat of intersect (deg)
!			PLON	REAL*4	east longitude of intersect (deg)
!			GOOD	LOG*1	.TRUE. if line tracing is successful
!
!
	COMMON /ITER/R,DLAT,DLON,RP,DLATP,DLONP,BR,BT,BP,B,SGN,DS,L
!
	INTEGER*2	YEAR,DAY,DIR,MAXS,L,BDIR,YEAR4
	REAL*4		TM,GLAT,GLONG,RKM,HALT,PLAT,PLON,DLAT,DLON,
     &			RP,DLATP,DLONP,BR,BT,BP,B,SGN,C1,RA,
     &			PINTER,A1,A2,A3,A4,A5,A6,A7,R,H,HP,HPP,DLATPP,
     &			DLONPP,SINGML,DS,ALT,RADIUS,CT
	LOGICAL*1	GOOD
!
	DATA C1/.0067397/, RA/6378.16/, MAXS/500/
	PINTER(A1,A2,A3,A4,A5,A6,A7) = ((A2-A3)*(A7-A2)*(A7-A3)*A4-
     &	   (A1-A3)*(A7-A1)*(A7-A3)*A5+(A1-A2)*(A7-A1)*(A7-A2)*A6)/
     &     ((A1-A2)*(A1-A3)*(A2-A3))
!
!---need time in years
	TM = YEAR4(YEAR) + DAY/(366.-MIN0(MOD(YEAR,4),1))
!
	SINGML = .98*SIND(GLAT) + .199*COSD(GLAT)*COSD(GLONG+69.)
	DS = .06*RKM/(1.-SINGML**2)-370.
	IF (DS.GT.3.E3) DS = 3.E3
	ALT = RKM - RADIUS(GLAT)
	IF (DIR.LT.1.AND.DS.GT.ABS(ALT-HALT)/20.) DS = ABS(ALT-HALT)/20.
	IF (ALT+DIR*HALT.LT.0.) THEN
	    GOOD = .FALSE.
	    RETURN
	ENDIF
	DLAT = GLAT
	DLON = GLONG
	R = RKM
	L = 0
	IF (DIR.EQ.0) THEN
	    BDIR = 1
	ELSE
	    BDIR = DIR
	ENDIF
!
 24	CT = SIND(DLAT)
	CALL MAGSAT(YEAR,DAY,R,DLAT,DLON,BR,BT,BP,B)
	IF (L.EQ.0) SGN = SIGN(1.,BR*BDIR)
	L = L + 1
	IF (L.GT.MAXS) THEN
	    GOOD = .FALSE.
	    RETURN
	ENDIF
	DLATPP = DLATP
	DLONPP = DLONP
	CALL ITERAT
	HPP = HP
	HP = RP - RA/SQRT(1.+C1*CT*CT)  
	H = R - RA/SQRT(1.+C1*SIND(DLAT)**2)
	IF (DIR.EQ.0) THEN
	    IF (H.LT.HALT) GO TO 24
	ELSE
	    IF (R.GT.RP) GO TO 24
	    IF (H.GT.HALT) GO TO 24
	ENDIF
	IF (HPP.EQ.HP.OR.HPP.EQ.H.OR.HP.EQ.H) THEN
	    GOOD = .FALSE.
	    RETURN
	ENDIF
	PLAT = PINTER(HPP,HP,H,DLATPP,DLATP,DLAT,HALT)
	PLON = PINTER(HPP,HP,H,DLONPP,DLONP,DLON,HALT)
	PLON = AMOD(PLON,360.)
!
	GOOD = .TRUE.
	RETURN
!
	END
!
!
!
	SUBROUTINE ITERAT
!
!  field line integration using a 4-point Adams formula
!
	COMMON /ITER/Y,YOLD,BR,BT,BP,B,SGN,DS,L
!
	REAL*4		Y(3),YOLD(3),BR,BT,BP,B,SGN,DS,YP(3,4),
     &			FAC,D2,D6,D12,D24
	INTEGER*2	L,I,J
!
	YP(1,4) = SGN*(BR/B)
	FAC = SGN*57.2957795/(B*Y(1))
	YP(2,4) = -BT*FAC
	YP(3,4) = BP*FAC/COSD(Y(2))
!
	IF (L.GT.7) THEN
	    DO 12 I=1,3
	    YOLD(I) = Y(I)
    	    Y(I) = YOLD(I) + D24*(55.*YP(I,4)
     &			-59.*YP(I,3)+37.*YP(I,2)-9.*YP(I,1))
	    DO 10 J=1,3
	    YP(I,J) = YP(I,J+1)
 10	    CONTINUE
 12	    CONTINUE
	    RETURN
	ENDIF
!
	DO 88 I=1,3
	GO TO (51,52,53,54,55,56,57),L
 51	D2 = DS/2.
	D6 = DS/6.
	D12 = DS/12.
	D24 = DS/24.
	YP(I,1) = YP(I,4)
	YOLD(I) = Y(I)
	Y(I) = YOLD(I) + DS*YP(I,1)
	GO TO 88
 52	YP(I,2) = YP(I,4)
	Y(I) = YOLD(I) + D2*(YP(I,2)+YP(I,1))
	GO TO 88
 53	Y(I) = YOLD(I) + D6*(2.*YP(I,4)+YP(I,2)+3.*YP(I,1))
	GO TO 88
 54	YP(I,2) = YP(I,4)
	YOLD(I) = Y(I)
	Y(I) = YOLD(I) + D2*(3.*YP(I,2)-YP(I,1))
	GO TO 88
 55	Y(I) = YOLD(I) + D12*(5.*YP(I,4)+8.*YP(I,2)-YP(I,1))
	GO TO 88
 56	YP(I,3) = YP(I,4)
	YOLD(I) = Y(I)
    	Y(I) = YOLD(I) + D12*(23.*YP(I,3)-16.*YP(I,2)+5.*YP(I,1))
	GO TO 88
 57	Y(I) = YOLD(I) + D24*(9.*YP(I,4)+19.*YP(I,3)-5.*YP(I,2)+YP(I,1))
 88	CONTINUE
	RETURN
!
	END
!
!
!
	BLOCK DATA CGMCRD
!
	COMMON /CGMCRDS/CGMC
!
	REAL*4		CGMC(72,153)
	INTEGER*2	I
!
!  column    1 data, 36 LAT,LNG pairs
	DATA (CGMC(I,  1),I=1,72)/
     &  -72.60,  23.80, -72.90,  24.70, -73.20,  25.30, -73.50,  25.70,
     &  -73.80,  25.70, -74.10,  25.40, -74.30,  24.70, -74.50,  24.30,
     &  -74.70,  23.80, -75.00,  23.00, -75.20,  22.30, -75.40,  21.00,
     &  -75.70,  20.00, -75.90,  18.00, -76.20,  15.50, -76.00,  13.50,
     &  -75.90,  12.00, -75.70,  11.20, -75.30,  10.80, -74.80,  10.50,
     &  -74.50,  10.40, -73.90,  10.50, -73.50,  10.80, -73.10,  11.20,
     &  -72.80,  11.60, -72.50,  12.20, -72.20,  12.50, -72.00,  13.10,
     &  -71.80,  14.20, -71.70,  16.00, -71.70,  17.00, -71.70,  18.00,
     &  -71.80,  19.50, -71.90,  21.00, -72.10,  21.40, -72.30,  22.60/
!  column    2 data, 36 LAT,LNG pairs
	DATA (CGMC(I,  2),I=1,72)/
     &  -71.20,  27.30, -71.70,  28.80, -72.30,  30.10, -72.90,  31.00,
     &  -73.50,  31.60, -74.20,  31.80, -74.80,  31.60, -75.50,  30.90,
     &  -76.10,  29.70, -76.60,  28.10, -77.10,  25.90, -77.50,  23.30,
     &  -77.80,  20.40, -77.80,  17.30, -77.70,  14.00, -77.40,  11.00,
     &  -76.90,   8.20, -76.60,   6.40, -76.00,   5.00, -75.30,   4.50,
     &  -74.50,   4.30, -73.80,   4.50, -73.10,   4.90, -72.40,   5.70,
     &  -71.80,   6.70, -71.30,   7.80, -70.80,   9.20, -70.40,  10.70,
     &  -70.10,  12.40, -69.90,  14.50, -69.80,  16.00, -69.80,  17.90,
     &  -69.90,  19.90, -70.10,  21.90, -70.40,  23.80, -70.70,  25.60/
!  column    3 data, 36 LAT,LNG pairs
	DATA (CGMC(I,  3),I=1,72)/
     &  -69.80,  30.70, -70.50,  32.90, -71.30,  34.90, -72.20,  36.50,
     &  -73.10,  37.80, -74.10,  38.60, -75.10,  38.90, -76.10,  38.50,
     &  -77.00,  37.20, -77.90,  35.00, -78.70,  31.70, -79.30,  27.40,
     &  -79.70,  22.20, -79.80,  16.60, -79.50,  11.00, -79.00,   6.00,
     &  -78.40,   2.00, -77.60, 359.20, -76.70, 357.50, -75.60, 356.70,
     &  -74.60, 356.70, -73.50, 357.30, -72.50, 358.40, -71.90, 359.90,
     &  -70.70,   1.70, -69.90,   3.70, -69.20,   5.80, -68.60,   8.10,
     &  -68.20,  10.50, -67.90,  13.00, -67.80,  15.50, -67.80,  18.10,
     &  -67.90,  20.70, -68.20,  23.30, -68.60,  25.90, -69.20,  28.30/
!  column    4 data, 36 LAT,LNG pairs
	DATA (CGMC(I,  4),I=1,72)/
     &  -68.40,  33.70, -69.30,  36.60, -70.30,  39.20, -71.40,  41.60,
     &  -72.50,  43.70, -73.80,  45.30, -75.10,  46.20, -76.40,  46.40,
     &  -77.70,  45.50, -79.00,  43.30, -80.10,  39.20, -81.00,  33.00,
     &  -81.60,  24.70, -81.80,  15.30, -81.50,   7.00, -80.70, 358.70,
     &  -79.70, 353.40, -78.50, 350.20, -77.10, 348.70, -75.70, 348.40,
     &  -74.40, 349.20, -73.00, 350.60, -71.70, 352.50, -70.50, 354.80,
     &  -69.40, 357.30, -68.40,   0.00, -67.50,   2.90, -66.80,   5.80,
     &  -66.30,   8.80, -66.00,  11.90, -65.80,  15.00, -65.80,  18.10,
     &  -66.00,  21.20, -66.40,  24.40, -66.90,  27.60, -67.60,  30.70/
!  column    5 data, 36 LAT,LNG pairs
	DATA (CGMC(I,  5),I=1,72)/
     &  -67.00,  36.20, -68.00,  39.80, -69.20,  43.10, -70.40,  46.30,
     &  -71.80,  49.10, -73.30,  51.50, -74.80,  53.40, -76.40,  54.50,
     &  -78.10,  54.50, -79.80,  52.90, -81.30,  48.80, -82.70,  41.00,
     &  -83.60,  28.80, -83.80,  13.20, -83.30, 358.70, -82.10, 348.40,
     &  -80.70, 342.60, -79.00, 339.90, -77.30, 339.30, -75.60, 340.10,
     &  -73.90, 341.90, -72.30, 344.30, -70.70, 347.10, -69.30, 350.20,
     &  -68.00, 353.50, -66.80, 356.80, -65.90,   0.30, -65.00,   3.80,
     &  -64.40,   7.40, -64.00,  10.90, -63.90,  14.50, -63.90,  18.00,
     &  -64.20,  21.60, -64.60,  25.30, -65.20,  29.00, -66.00,  32.60/
!  column    6 data, 36 LAT,LNG pairs
	DATA (CGMC(I,  6),I=1,72)/
     &  -65.50,  38.50, -66.70,  42.60, -68.00,  46.60, -69.40,  50.50,
     &  -71.00,  54.10, -72.60,  57.40, -74.40,  60.20, -76.20,  62.50,
     &  -78.20,  63.80, -80.20,  63.70, -82.20,  61.00, -84.10,  53.20,
     &  -85.50,  36.10, -85.80,   8.90, -84.90, 345.90, -83.20, 334.20,
     &  -81.20, 329.60, -79.20, 328.70, -77.10, 329.80, -75.10, 332.10,
     &  -73.20, 335.10, -71.30, 338.50, -69.60, 342.20, -68.00, 346.10,
     &  -66.50, 350.10, -65.20, 354.10, -64.00, 358.10, -63.10,   2.10,
     &  -62.50,   6.10, -62.00,  10.00, -61.90,  14.00, -62.00,  17.90,
     &  -62.30,  21.90, -62.80,  26.00, -63.60,  30.10, -64.50,  34.30/
!  column    7 data, 36 LAT,LNG pairs
	DATA (CGMC(I,  7),I=1,72)/
     &  -64.10,  40.40, -65.40,  45.10, -66.80,  49.70, -68.40,  54.20,
     &  -70.00,  58.60, -71.80,  62.70, -73.70,  66.50, -75.80,  70.00,
     &  -78.00,  73.00, -80.30,  75.10, -82.70,  75.50, -85.10,  71.60,
     &  -87.10,  55.20, -87.80, 358.20, -86.00, 323.60, -83.70, 316.00,
     &  -81.30, 315.50, -78.90, 317.50, -76.60, 320.70, -74.40, 324.60,
     &  -72.30, 328.80, -70.20, 333.20, -68.30, 337.80, -66.60, 342.40,
     &  -64.90, 347.10, -63.50, 351.70, -62.20, 356.20, -61.20,   0.60,
     &  -60.50,   4.90, -60.00,   9.20, -59.90,  13.50, -60.00,  17.80,
     &  -60.40,  22.10, -61.10,  26.60, -62.00,  31.10, -63.00,  35.70/
!  column    8 data, 36 LAT,LNG pairs
	DATA (CGMC(I,  8),I=1,72)/
     &  -62.80,  42.10, -64.20,  47.30, -65.60,  52.50, -67.20,  57.60,
     &  -69.00,  62.60, -70.90,  67.50, -72.90,  72.30, -75.10,  77.00,
     &  -77.50,  81.70, -80.00,  86.40, -82.70,  91.00, -85.40,  95.50,
     &  -88.10, 100.60, -88.90, 285.30, -86.30, 291.90, -83.50, 296.70,
     &  -80.90, 301.80, -78.30, 307.10, -75.80, 312.40, -73.40, 317.70,
     &  -71.20, 323.10, -69.00, 328.50, -67.00, 333.90, -65.10, 339.20,
     &  -63.30, 344.40, -61.80, 349.50, -60.40, 354.40, -59.30, 359.20,
     &  -58.50,   3.90, -58.10,   8.50, -57.90,  13.00, -58.10,  17.60,
     &  -58.60,  22.30, -59.40,  27.00, -60.40,  31.90, -61.50,  37.00/
!  column    9 data, 36 LAT,LNG pairs
	DATA (CGMC(I,  9),I=1,72)/
     &  -61.40,  43.70, -62.90,  49.30, -64.40,  55.00, -66.10,  60.70,
     &  -67.90,  66.30, -69.80,  71.90, -72.00,  77.50, -74.30,  83.40,
     &  -76.80,  89.60, -79.40,  96.60, -82.20, 105.40, -84.90, 118.70,
     &  -87.30, 149.30, -87.60, 226.10, -85.40, 264.90, -82.70, 280.10,
     &  -80.00, 290.00, -77.30, 297.90, -74.70, 305.00, -72.20, 311.60,
     &  -69.90, 318.00, -67.60, 324.20, -65.50, 330.30, -63.50, 336.20,
     &  -61.70, 342.00, -60.00, 347.50, -58.60, 352.90, -57.40, 358.00,
     &  -56.60,   3.00, -56.10,   7.80, -55.90,  12.60, -56.20,  17.40,
     &  -56.80,  22.30, -57.70,  27.40, -58.80,  32.60, -60.00,  38.10/
!  column   10 data, 36 LAT,LNG pairs
	DATA (CGMC(I, 10),I=1,72)/
     &  -60.10,  45.00, -61.60,  51.10, -63.20,  57.30, -64.90,  63.50,
     &  -66.80,  69.60, -68.70,  75.80, -70.90,  82.20, -73.30,  89.00,
     &  -75.80,  96.60, -78.50, 105.60, -81.20, 117.40, -83.80, 135.60,
     &  -85.60, 168.30, -85.60, 215.20, -83.90, 248.70, -81.40, 267.70,
     &  -78.70, 280.30, -76.00, 290.20, -73.40, 298.60, -70.90, 306.30,
     &  -68.50, 313.50, -66.20, 320.40, -64.00, 327.10, -61.90, 333.60,
     &  -60.00, 339.80, -58.30, 345.80, -56.70, 351.50, -55.50, 357.00,
     &  -54.60,   2.20, -54.10,   7.20, -54.00,  12.00, -54.30,  17.20,
     &  -55.00,  22.30, -56.00,  27.70, -57.20,  33.30, -58.60,  39.10/
!  column   11 data, 36 LAT,LNG pairs
	DATA (CGMC(I, 11),I=1,72)/
     &  -58.80,  46.30, -60.40,  52.80, -62.00,  59.40, -63.80,  66.00,
     &  -65.60,  72.70, -67.60,  79.40, -69.80,  86.40, -72.20,  94.00,
     &  -74.70, 102.60, -77.40, 113.10, -80.00, 126.90, -82.20, 146.80,
     &  -83.70, 176.10, -83.60, 211.00, -82.00, 239.30, -79.80, 258.70,
     &  -77.20, 272.70, -74.60, 283.80, -72.00, 293.20, -69.40, 301.70,
     &  -67.00, 309.60, -64.60, 317.00, -62.40, 324.20, -60.30, 331.10,
     &  -58.30, 337.80, -56.50, 344.20, -54.90, 350.20, -53.50, 356.00,
     &  -52.60,   1.40, -52.10,   6.70, -52.00,  11.80, -52.40,  17.00,
     &  -53.20,  22.30, -54.40,  27.90, -55.70,  33.80, -57.20,  39.90/
!  column   12 data, 36 LAT,LNG pairs
	DATA (CGMC(I, 12),I=1,72)/
     &  -57.50,  47.50, -59.20,  54.40, -60.90,  61.40, -62.60,  68.40,
     &  -64.40,  75.40, -66.40,  82.60, -68.60,  90.20, -70.90,  98.40,
     &  -73.50, 107.90, -76.10, 119.40, -78.50, 134.20, -80.50, 154.10,
     &  -81.60, 180.20, -81.40, 208.80, -80.10, 233.50, -78.00, 252.30,
     &  -75.50, 266.70, -72.90, 278.50, -70.40, 288.60, -67.80, 297.60,
     &  -65.40, 306.00, -63.00, 314.00, -60.80, 321.60, -58.60, 328.90,
     &  -56.60, 336.00, -54.70, 342.70, -53.00, 349.10, -51.60, 355.10,
     &  -50.60,   0.70, -50.10,   6.10, -50.00,  11.40, -50.50,  16.80,
     &  -51.50,  22.30, -52.80,  28.10, -54.30,  34.20, -55.90,  40.70/
!  column   13 data, 36 LAT,LNG pairs
	DATA (CGMC(I, 13),I=1,72)/
     &  -56.30,  48.50, -58.00,  55.80, -59.70,  63.20, -61.40,  70.60,
     &  -63.20,  78.00, -65.10,  85.60, -67.30,  93.60, -69.60, 102.30,
     &  -72.10, 112.40, -74.60, 124.60, -76.90, 139.80, -78.70, 159.30,
     &  -79.60, 182.80, -79.30, 207.50, -78.00, 229.50, -76.10, 247.50,
     &  -73.70, 262.00, -71.20, 274.10, -68.60, 284.60, -66.10, 294.10,
     &  -63.70, 302.90, -61.30, 311.20, -59.10, 319.20, -56.90, 326.90,
     &  -54.80, 334.30, -52.90, 341.40, -51.20, 348.00, -49.70, 354.30,
     &  -48.60,   0.10, -48.10,   5.70, -48.10,  11.10, -48.70,  16.50,
     &  -49.70,  22.20, -51.20,  28.20, -52.80,  34.70, -54.60,  41.40/
!  column   14 data, 36 LAT,LNG pairs
	DATA (CGMC(I, 14),I=1,72)/
     &  -55.10,  49.50, -56.80,  57.10, -58.50,  64.90, -60.20,  72.60,
     &  -61.90,  80.30, -63.80,  88.30, -65.90,  96.60, -68.20, 105.80,
     &  -70.70, 116.30, -73.10, 128.90, -75.20, 144.30, -76.80, 163.00,
     &  -77.50, 184.50, -77.20, 206.50, -75.90, 226.70, -74.10, 243.80,
     &  -71.80, 258.20, -69.40, 270.40, -66.90, 281.20, -64.40, 291.00,
     &  -62.00, 300.10, -59.60, 308.70, -57.40, 317.00, -55.20, 325.10,
     &  -53.10, 332.80, -51.10, 340.10, -49.30, 347.00, -47.80, 353.50,
     &  -46.70, 359.50, -46.10,   5.20, -46.10,  10.70, -46.80,  16.30,
     &  -48.00,  22.20, -49.60,  28.40, -51.40,  35.00, -53.30,  42.10/
!  column   15 data, 36 LAT,LNG pairs
	DATA (CGMC(I, 15),I=1,72)/
     &  -53.90,  50.50, -55.70,  58.40, -57.30,  66.40, -59.00,  74.50,
     &  -60.70,  82.50, -62.50,  90.80, -64.60,  99.40, -66.80, 108.90,
     &  -69.10, 119.80, -71.40, 132.60, -73.40, 147.90, -74.80, 165.80,
     &  -75.40, 185.70, -75.00, 205.90, -73.80, 224.50, -72.00, 240.90,
     &  -69.90, 255.00, -67.50, 267.30, -65.00, 278.30, -62.60, 288.30,
     &  -60.20, 297.60, -57.80, 306.50, -55.60, 315.00, -53.40, 323.30,
     &  -51.30, 331.30, -49.30, 338.90, -47.50, 346.10, -45.90, 352.80,
     &  -44.70, 359.00, -44.10,   4.80, -44.20,  10.40, -45.00,  16.10,
     &  -46.30,  22.10, -48.10,  28.50, -50.00,  35.40, -52.00,  42.70/
!  column   16 data, 36 LAT,LNG pairs
	DATA (CGMC(I, 16),I=1,72)/
     &  -52.70,  51.30, -54.50,  59.60, -56.20,  67.90, -57.80,  76.30,
     &  -59.40,  84.60, -61.20,  93.00, -63.10, 101.90, -65.30, 111.70,
     &  -67.60, 122.80, -69.70, 135.70, -71.50, 150.80, -72.80, 168.00,
     &  -73.20, 186.60, -72.80, 205.30, -71.70, 222.90, -70.00, 238.60,
     &  -67.90, 252.40, -65.60, 264.70, -63.10, 275.70, -60.70, 285.90,
     &  -58.40, 295.40, -56.10, 304.40, -53.80, 313.20, -51.70, 321.70,
     &  -49.60, 329.90, -47.60, 337.80, -45.60, 345.20, -44.00, 352.10,
     &  -42.80, 358.50, -42.20,   4.40, -42.30,  10.10, -43.20,  15.90,
     &  -44.70,  22.00, -46.60,  28.60, -48.70,  35.70, -50.80,  43.30/
!  column   17 data, 36 LAT,LNG pairs
	DATA (CGMC(I, 17),I=1,72)/
     &  -51.60,  52.20, -53.40,  60.70, -55.10,  69.40, -56.60,  78.00,
     &  -58.10,  86.50, -59.80,  95.20, -61.70, 104.30, -63.80, 114.20,
     &  -65.90, 125.40, -68.00, 138.40, -69.60, 153.20, -70.70, 169.80,
     &  -71.10, 187.30, -70.60, 204.90, -69.50, 221.50, -67.90, 236.70,
     &  -65.80, 250.20, -63.60, 262.40, -61.20, 273.50, -58.80, 283.70,
     &  -56.50, 293.30, -54.20, 302.60, -52.00, 311.50, -49.90, 320.20,
     &  -47.80, 328.70, -45.80, 336.80, -43.80, 344.40, -42.10, 351.50,
     &  -40.80, 358.00, -40.20,   4.10, -40.40,   9.90, -41.30,  15.80,
     &  -43.00,  22.00, -45.10,  28.70, -47.30,  36.00, -49.50,  43.90/
!  column   18 data, 36 LAT,LNG pairs
	DATA (CGMC(I, 18),I=1,72)/
     &  -50.50,  53.00, -52.30,  61.80, -53.90,  70.70, -55.40,  79.60,
     &  -56.80,  88.30, -58.40,  97.20, -60.20, 106.40, -62.20, 116.50,
     &  -64.20, 127.80, -66.20, 140.70, -67.70, 155.30, -68.60, 171.20,
     &  -68.90, 187.90, -68.50, 204.60, -67.40, 220.40, -65.80, 235.00,
     &  -63.80, 248.30, -61.60, 260.40, -59.30, 271.50, -56.90, 281.80,
     &  -54.60, 291.60, -52.40, 300.80, -50.20, 309.90, -48.10, 318.80,
     &  -46.10, 327.40, -44.00, 335.80, -42.00, 343.60, -40.30, 350.90,
     &  -38.90, 357.60, -38.30,   3.70, -38.50,   9.60, -39.60,  15.60,
     &  -41.40,  21.90, -43.60,  28.80, -46.00,  36.30, -48.30,  44.40/
!  column   19 data, 36 LAT,LNG pairs
	DATA (CGMC(I, 19),I=1,72)/
     &  -49.30,  53.80, -51.20,  62.90, -52.80,  72.00, -54.20,  81.10,
     &  -55.50,  90.00, -57.00,  99.00, -58.70, 108.40, -60.60, 118.60,
     &  -62.50, 129.90, -64.30, 142.80, -65.70, 157.00, -66.50, 172.40,
     &  -66.70, 188.40, -66.20, 204.30, -65.20, 219.50, -63.60, 233.70,
     &  -61.70, 246.70, -59.60, 258.70, -57.30, 269.70, -55.00, 280.00,
     &  -52.70, 289.80, -50.50, 299.20, -48.40, 308.40, -46.30, 317.50,
     &  -44.30, 326.30, -42.30, 334.80, -40.30, 342.90, -38.40, 350.30,
     &  -37.00, 357.10, -36.40,   3.40, -36.60,   9.40, -37.80,  15.50,
     &  -39.80,  21.90, -42.20,  28.90, -44.70,  36.70, -47.10,  45.00/
!  column   20 data, 36 LAT,LNG pairs
	DATA (CGMC(I, 20),I=1,72)/
     &  -48.20,  54.60, -50.10,  63.90, -51.60,  73.30, -53.00,  82.60,
     &  -54.20,  91.70, -55.60, 100.80, -57.20, 110.30, -58.90, 120.50,
     &  -60.80, 131.90, -62.40, 144.60, -63.70, 158.60, -64.40, 173.50,
     &  -64.50, 188.80, -64.00, 204.00, -63.00, 218.70, -61.50, 232.50,
     &  -59.60, 245.30, -57.50, 257.10, -55.30, 268.10, -53.00, 278.40,
     &  -50.80, 288.30, -48.60, 297.80, -46.50, 307.10, -44.50, 316.20,
     &  -42.50, 325.20, -40.50, 333.90, -38.50, 342.10, -36.60, 349.80,
     &  -35.20, 356.70, -34.50,   3.10, -34.80,   9.20, -36.10,  15.30,
     &  -38.20,  21.90, -40.70,  29.10, -43.40,  37.00, -46.00,  45.50/
!  column   21 data, 36 LAT,LNG pairs
	DATA (CGMC(I, 21),I=1,72)/
     &  -47.10,  55.30, -49.00,  64.90, -50.50,  74.50, -51.80,  84.00,
     &  -52.90,  93.20, -54.10, 102.50, -55.60, 112.00, -57.30, 122.30,
     &  -59.00, 133.60, -60.50, 146.20, -61.70, 159.90, -62.30, 174.40,
     &  -62.30, 189.20, -61.80, 203.80, -60.80, 218.00, -59.30, 231.40,
     &  -57.50, 244.00, -55.50, 255.70, -53.30, 266.70, -51.00, 277.00,
     &  -48.80, 286.90, -46.70, 296.40, -44.70, 305.80, -42.70, 315.00,
     &  -40.70, 324.10, -38.80, 333.00, -36.70, 341.40, -34.80, 349.20,
     &  -33.30, 356.30, -32.60,   2.80, -32.90,   9.00, -34.30,  15.30,
     &  -36.60,  21.90, -39.30,  29.30, -42.10,  37.30, -44.80,  46.10/
!  column   22 data, 36 LAT,LNG pairs
	DATA (CGMC(I, 22),I=1,72)/
     &  -45.90,  56.10, -47.90,  65.90, -49.30,  75.70, -50.50,  85.30,
     &  -51.60,  94.70, -52.70, 104.00, -54.00, 113.60, -55.60, 123.90,
     &  -57.20, 135.20, -58.60, 147.70, -59.60, 161.10, -60.10, 175.20,
     &  -60.10, 189.50, -59.60, 203.60, -58.60, 217.30, -57.20, 230.50,
     &  -55.40, 242.90, -53.40, 254.50, -51.20, 265.40, -49.00, 275.70,
     &  -46.90, 285.60, -44.80, 295.10, -42.80, 304.60, -40.90, 313.90,
     &  -39.00, 323.10, -37.00, 332.10, -35.00, 340.70, -33.00, 348.70,
     &  -31.50, 356.00, -30.70,   2.60, -31.10,   8.90, -32.60,  15.20,
     &  -35.00,  22.00, -37.90,  29.50, -40.80,  37.70, -43.60,  46.60/
!  column   23 data, 36 LAT,LNG pairs
	DATA (CGMC(I, 23),I=1,72)/
     &  -44.80,  56.80, -46.70,  66.80, -48.20,  76.80, -49.30,  86.60,
     &  -50.20,  96.10, -51.20, 105.50, -52.40, 115.10, -53.80, 125.40,
     &  -55.30, 136.70, -56.60, 149.00, -57.50, 162.20, -58.00, 175.90,
     &  -57.90, 189.70, -57.40, 203.40, -56.40, 216.80, -55.00, 229.60,
     &  -53.30, 241.90, -51.30, 253.40, -49.20, 264.20, -47.00, 274.50,
     &  -44.90, 284.30, -42.90, 294.00, -40.90, 303.40, -39.00, 312.80,
     &  -37.20, 322.10, -35.30, 331.30, -33.20, 340.00, -31.30, 348.20,
     &  -29.60, 355.60, -28.80,   2.30, -29.20,   8.70, -30.90,  15.20,
     &  -33.50,  22.10, -36.50,  29.70, -39.60,  38.10, -42.40,  47.20/
!  column   24 data, 36 LAT,LNG pairs
	DATA (CGMC(I, 24),I=1,72)/
     &  -43.60,  57.60, -45.60,  67.70, -47.00,  77.90, -48.00,  87.90,
     &  -48.80,  97.50, -49.70, 106.90, -50.80, 116.60, -52.10, 126.80,
     &  -53.50, 138.00, -54.60, 150.20, -55.50, 163.20, -55.80, 176.50,
     &  -55.70, 190.00, -55.10, 203.30, -54.20, 216.30, -52.90, 228.90,
     &  -51.20, 241.00, -49.20, 252.40, -47.10, 263.10, -45.00, 273.40,
     &  -42.90, 283.20, -40.90, 292.90, -39.00, 302.40, -37.20, 311.80,
     &  -35.40, 321.20, -33.50, 330.40, -31.50, 339.40, -29.50, 347.70,
     &  -27.80, 355.20, -27.00,   2.10, -27.40,   8.60, -29.20,  15.20,
     &  -31.90,  22.10, -35.10,  30.00, -38.30,  38.50, -41.20,  47.80/
!  column   25 data, 36 LAT,LNG pairs
	DATA (CGMC(I, 25),I=1,72)/
     &  -42.40,  58.30, -44.40,  68.60, -45.70,  79.00, -46.70,  89.10,
     &  -47.40,  98.80, -48.10, 108.30, -49.10, 117.90, -50.30, 128.10,
     &  -51.60, 139.30, -52.60, 151.30, -53.30, 164.00, -53.60, 177.10,
     &  -53.40, 190.20, -52.90, 203.10, -52.00, 215.80, -50.70, 228.20,
     &  -49.10, 240.10, -47.10, 251.40, -45.10, 262.10, -42.90, 272.30,
     &  -40.90, 282.20, -38.90, 291.80, -37.10, 301.40, -35.30, 310.90,
     &  -33.60, 320.30, -31.80, 329.70, -29.80, 338.70, -27.70, 347.20,
     &  -26.00, 354.90, -25.10,   1.90, -25.60,   8.60, -27.50,  15.20,
     &  -30.30,  22.40, -33.70,  30.30, -37.00,  38.90, -40.00,  48.40/
!  column   26 data, 36 LAT,LNG pairs
	DATA (CGMC(I, 26),I=1,72)/
     &  -41.20,  59.10, -43.20,  69.50, -44.50,  80.00, -45.30,  90.20,
     &  -45.90, 100.00, -46.50, 109.60, -47.40, 119.20, -48.50, 129.40,
     &  -49.60, 140.40, -50.60, 152.30, -51.20, 164.80, -51.40, 177.60,
     &  -51.20, 190.30, -50.70, 203.00, -49.80, 215.40, -48.50, 227.60,
     &  -46.90, 239.40, -45.00, 250.60, -43.00, 261.20, -40.90, 271.40,
     &  -38.90, 281.20, -37.00, 290.90, -35.20, 300.40, -33.50, 310.00,
     &  -31.80, 319.50, -30.00, 328.90, -28.10, 338.10, -26.00, 346.70,
     &  -24.20, 354.50, -23.20,   1.70, -23.70,   8.50, -25.80,  15.30,
     &  -28.80,  22.60, -32.30,  30.60, -35.70,  39.40, -38.70,  49.00/
!  column   27 data, 36 LAT,LNG pairs
	DATA (CGMC(I, 27),I=1,72)/
     &  -40.00,  59.80, -41.90,  70.40, -43.20,  81.00, -44.00,  91.30,
     &  -44.40, 101.20, -44.90, 110.80, -45.60, 120.60, -46.60, 130.50,
     &  -47.70, 141.50, -48.60, 153.20, -49.10, 165.50, -49.20, 178.00,
     &  -49.00, 190.50, -48.40, 202.80, -47.60, 215.00, -46.40, 227.00,
     &  -44.80, 238.70, -42.90, 249.80, -40.90, 260.40, -38.80, 270.50,
     &  -36.80, 280.30, -35.00, 290.00, -33.20, 299.60, -31.60, 309.10,
     &  -30.00, 318.70, -28.30, 328.20, -26.30, 337.40, -24.20, 346.20,
     &  -22.30, 354.20, -21.40,   1.50, -21.90,   8.50, -24.00,  15.50,
     &  -27.20,  22.90, -30.80,  31.00, -34.30,  39.90, -37.40,  49.60/
!  column   28 data, 36 LAT,LNG pairs
	DATA (CGMC(I, 28),I=1,72)/
     &  -38.70,  60.60, -40.60,  71.20, -41.80,  81.90, -42.50,  92.40,
     &  -42.90, 102.30, -43.20, 111.90, -43.90, 121.50, -44.70, 131.60,
     &  -45.70, 142.50, -46.50, 154.10, -47.00, 166.20, -47.00, 178.50,
     &  -46.70, 190.60, -46.20, 202.70, -45.30, 214.70, -44.20, 226.50,
     &  -42.70, 238.00, -40.80, 249.10, -38.70, 259.60, -36.70, 269.70,
     &  -34.80, 279.50, -33.00, 289.10, -31.30, 298.80, -29.70, 308.30,
     &  -28.10, 317.90, -26.50, 327.50, -24.60, 336.80, -22.50, 345.70,
     &  -20.50, 353.90, -19.50,   1.40, -20.10,   8.50, -22.30,  15.60,
     &  -25.70,  23.20, -29.40,  31.40, -33.00,  40.50, -36.10,  50.20/
!  column   29 data, 36 LAT,LNG pairs
	DATA (CGMC(I, 29),I=1,72)/
     &  -37.30,  61.30, -39.20,  72.10, -40.40,  82.90, -41.10,  93.40,
     &  -41.30, 103.40, -41.50, 113.00, -42.00, 122.60, -42.80, 132.60,
     &  -43.80, 143.40, -44.50, 154.90, -44.80, 166.80, -44.80, 178.80,
     &  -44.50, 190.70, -43.90, 202.60, -43.10, 214.30, -42.00, 226.00,
     &  -40.50, 237.40, -38.70, 248.40, -36.60, 258.80, -34.60, 268.90,
     &  -32.70, 278.70, -31.00, 288.40, -29.30, 298.00, -27.80, 307.60,
     &  -26.30, 317.20, -24.70, 326.80, -22.80, 336.30, -20.70, 345.30,
     &  -18.70, 353.60, -17.60,   1.20, -18.30,   8.50, -20.60,  15.80,
     &  -24.10,  23.50, -27.90,  31.90, -31.60,  41.10, -34.80,  50.90/
!  column   30 data, 36 LAT,LNG pairs
	DATA (CGMC(I, 30),I=1,72)/
     &  -35.90,  62.10, -37.80,  72.90, -39.00,  83.70, -39.50,  94.30,
     &  -39.70, 104.40, -39.80, 114.00, -40.20, 123.50, -40.90, 133.50,
     &  -41.70, 144.20, -42.40, 155.60, -42.70, 167.40, -42.60, 179.20,
     &  -42.20, 190.80, -41.70, 202.40, -40.90, 214.00, -39.80, 225.50,
     &  -38.40, 236.80, -36.50, 247.70, -34.50, 258.20, -32.50, 268.20,
     &  -30.60, 277.90, -28.90, 287.60, -27.40, 297.30, -25.90, 306.90,
     &  -24.40, 316.50, -22.90, 326.20, -21.10, 335.70, -18.90, 344.80,
     &  -16.90, 353.30, -15.70,   1.10, -16.40,   8.60, -18.90,  16.10,
     &  -22.50,  23.90, -26.40,  32.40, -30.20,  41.70, -33.40,  51.60/
!  column   31 data, 36 LAT,LNG pairs
	DATA (CGMC(I, 31),I=1,72)/
     &  -34.50,  62.80, -36.30,  73.70, -37.50,  84.60, -37.90,  95.20,
     &  -38.00, 105.30, -38.00, 114.90, -38.30, 124.50, -39.00, 134.40,
     &  -39.70, 145.00, -40.30, 156.30, -40.50, 167.90, -40.30, 179.50,
     &  -40.00, 190.90, -39.40, 202.30, -38.70, 213.70, -37.70, 225.10,
     &  -36.20, 236.30, -34.40, 247.20, -32.30, 257.50, -30.30, 267.50,
     &  -28.50, 277.20, -26.90, 286.90, -25.40, 296.60, -23.90, 306.20,
     &  -22.60, 315.90, -21.10, 325.60, -19.30, 335.20, -17.20, 344.40,
     &  -15.00, 353.00, -13.90,   1.00, -14.60,   8.70, -17.20,  16.40,
     &  -20.90,  24.40, -24.90,  33.00, -28.70,  42.30, -31.90,  52.30/
!  column   32 data, 36 LAT,LNG pairs
	DATA (CGMC(I, 32),I=1,72)/
     &  -32.90,  63.60, -34.80,  74.40, -35.90,  85.40, -36.30,  96.00,
     &  -36.30, 106.20, -36.20, 115.80, -36.40, 125.30, -37.00, 135.20,
     &  -37.70, 145.70, -38.20, 156.90, -38.30, 168.30, -38.10, 179.70,
     &  -37.70, 191.10, -37.20, 202.20, -36.50, 213.40, -35.50, 224.70,
     &  -34.10, 235.80, -32.20, 246.60, -30.20, 256.90, -28.20, 266.80,
     &  -26.40, 276.60, -24.80, 286.30, -23.40, 296.00, -22.00, 305.60,
     &  -20.70, 315.30, -19.20, 325.00, -17.50, 334.60, -15.40, 343.90,
     &  -13.20, 352.70, -11.90,   0.90, -12.70,   8.80, -15.50,  16.70,
     &  -19.30,  24.90, -23.30,  33.60, -27.10,  43.00, -30.40,  53.10/
!  column   33 data, 36 LAT,LNG pairs
	DATA (CGMC(I, 33),I=1,72)/
     &  -31.30,  64.30, -33.10,  75.20, -34.20,  86.10, -34.60,  96.80,
     &  -34.50, 106.90, -34.30, 116.60, -34.40, 126.10, -35.00, 135.90,
     &  -35.60, 146.40, -36.10, 157.40, -36.10, 168.70, -35.90, 180.00,
     &  -35.40, 191.00, -34.90, 202.00, -34.30, 213.10, -33.40, 224.30,
     &  -31.90, 235.40, -30.10, 246.10, -28.00, 256.30, -26.00, 266.20,
     &  -24.30, 275.90, -22.80, 285.70, -21.40, 295.40, -20.00, 305.10,
     &  -18.80, 314.80, -17.40, 324.50, -15.70, 334.10, -13.50, 343.50,
     &  -11.30, 352.40, -10.00,   0.80, -10.80,   9.00, -13.70,  17.10,
     &  -17.60,  25.40, -21.70,  34.30, -25.50,  43.80, -28.70,  53.80/
!  column   34 data, 36 LAT,LNG pairs
	DATA (CGMC(I, 34),I=1,72)/
     &  -29.60,  65.10, -31.40,  75.90, -32.40,  86.80, -32.80,  97.50,
     &  -32.70, 107.70, -32.40, 117.30, -32.50, 126.80, -32.90, 136.60,
     &  -33.60, 147.00, -34.00, 157.90, -34.00, 169.10, -33.60, 180.20,
     &  -33.20, 191.10, -32.70, 201.90, -32.10, 212.90, -31.20, 223.90,
     &  -29.80, 234.90, -27.90, 245.60, -25.80, 255.80, -23.80, 265.60,
     &  -22.20, 275.40, -20.70, 285.10, -19.40, 294.80, -18.10, 304.50,
     &  -16.80, 314.20, -15.50, 324.00, -13.80, 333.70, -11.70, 343.10,
     &   -9.40, 352.20,  -8.10,   0.80,  -8.90,   9.10, -12.00,  17.50,
     &  -15.90,  26.00, -20.00,  35.00, -23.80,  44.50, -27.00,  54.60/
!  column   35 data, 36 LAT,LNG pairs
	DATA (CGMC(I, 35),I=1,72)/
     &  -27.70,  65.80, -29.50,  76.60, -30.60,  87.40, -30.90,  98.10,
     &  -30.80, 108.30, -30.40, 118.00, -30.40, 127.40, -30.90, 137.20,
     &  -31.50, 147.50, -31.80, 158.40, -31.80, 169.40, -31.40, 180.30,
     &  -30.90, 191.10, -30.50, 201.80, -29.90, 212.60, -29.00, 223.60,
     &  -27.60, 234.50, -25.70, 245.10, -23.60, 255.30, -21.70, 265.10,
     &  -20.00, 274.80, -18.70, 284.50, -17.40, 294.30, -16.10, 304.00,
     &  -14.90, 313.70, -13.60, 323.50, -12.00, 333.20,  -9.90, 342.70,
     &   -7.50, 351.90,  -6.00,   0.70,  -7.00,   9.30, -10.20,  17.90,
     &  -14.20,  26.60, -18.20,  35.70, -22.00,  45.30, -25.20,  55.30/
!  column   36 data, 36 LAT,LNG pairs
	DATA (CGMC(I, 36),I=1,72)/
     &  -25.80,  66.50, -27.60,  77.20, -28.70,  88.00, -29.00,  98.60,
     &  -28.80, 108.80, -28.40, 118.60, -28.40, 128.00, -28.80, 137.70,
     &  -29.40, 148.00, -29.70, 158.80, -29.60, 169.70, -29.20, 180.50,
     &  -28.70, 191.10, -28.20, 201.70, -27.70, 212.40, -26.90, 223.30,
     &  -25.50, 234.10, -23.60, 244.70, -21.40, 254.80, -19.50, 264.60,
     &  -17.90, 274.30, -16.60, 284.00, -15.30, 293.80, -14.10, 303.60,
     &  -12.90, 313.30, -11.60, 323.10, -10.00, 332.80,  -8.00, 342.30,
     &   -5.60, 351.70,  -4.00,   0.70,  -5.10,   9.60,  -8.40,  18.40,
     &  -12.30,  27.30, -16.40,  36.40, -20.10,  46.00, -23.30,  56.10/
!  column   37 data, 36 LAT,LNG pairs
	DATA (CGMC(I, 37),I=1,72)/
     &  -23.70,  67.20, -25.50,  77.80, -26.70,  88.50, -27.00,  99.10,
     &  -26.80, 109.30, -26.40, 119.10, -26.30, 128.50, -26.70, 138.20,
     &  -27.20, 148.40, -27.50, 159.10, -27.40, 169.90, -26.90, 180.60,
     &  -26.40, 191.10, -26.00, 201.50, -25.50, 212.20, -24.80, 223.00,
     &  -23.40, 233.80, -21.40, 244.30, -19.30, 254.30, -17.30, 264.10,
     &  -15.80, 273.80, -14.50, 283.60, -13.30, 293.40, -12.10, 303.10,
     &  -10.90, 312.90,  -9.70, 322.60,  -8.10, 332.40,  -6.10, 342.00,
     &   -3.70, 351.40,  -1.40,   0.70,  -3.10,   9.80,  -6.60,  18.90,
     &  -10.40,  28.00, -14.30,  37.20, -18.00,  46.80, -21.20,  56.80/
!  column   38 data, 36 LAT,LNG pairs
	DATA (CGMC(I, 38),I=1,72)/
     &  -21.50,  67.80, -23.40,  78.30, -24.50,  88.90, -25.00,  99.50,
     &  -24.80, 109.80, -24.30, 119.50, -24.20, 129.00, -24.60, 138.70,
     &  -25.10, 148.80, -25.40, 159.40, -25.20, 170.20, -24.70, 180.70,
     &  -24.10, 191.00, -23.80, 201.40, -23.40, 211.90, -22.60, 222.70,
     &  -21.20, 233.40, -19.30, 243.90, -17.10, 253.90, -15.10, 263.70,
     &  -13.60, 273.40, -12.40, 283.10, -11.30, 292.90, -10.10, 302.70,
     &   -8.90, 312.50,  -7.60, 322.60,  -6.10, 332.00,  -4.10, 341.60,
     &   -1.60, 351.20,   0.00,   0.70,  -0.70,  10.10,  -4.70,  19.40,
     &   -8.30,  28.70, -12.10,  38.00, -15.70,  47.60, -18.90,  57.50/
!  column   39 data, 36 LAT,LNG pairs
	DATA (CGMC(I, 39),I=1,72)/
     &  -19.00,  68.40, -21.00,  78.80, -22.30,  89.30, -22.80,  99.90,
     &  -22.60, 110.10, -22.20, 119.90, -22.10, 129.40, -22.40, 139.10,
     &  -23.00, 149.20, -23.20, 159.70, -23.00, 170.30, -22.40, 180.80,
     &  -21.90, 191.00, -21.50, 201.30, -21.20, 211.70, -20.50, 222.40,
     &  -19.10, 233.10, -17.10, 243.50, -14.90, 253.50, -12.90, 263.20,
     &  -11.50, 272.90, -10.40, 282.70,  -9.20, 292.50,  -8.00, 302.30,
     &   -6.80, 312.10,  -5.50, 321.90,  -3.90, 331.60,  -4.10, 341.60,
     &   -1.60, 351.20,   0.00,   0.70,   0.00,   0.00,   0.00,   0.00,
     &    0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00,   0.00/
!  column   40 data, 36 LAT,LNG pairs
	DATA (CGMC(I, 40),I=1,72)/
     &    3.30,  73.80,   7.30,  82.50,  10.80,  91.60,  13.70, 101.20,
     &   15.90, 111.10,  17.30, 121.30,  17.70, 131.60,  17.40, 141.80,
     &   16.90, 151.70,  16.50, 161.40,  16.40, 171.10,  16.60, 180.90,
     &   16.60, 190.80,  16.50, 200.60,  16.30, 210.20,  16.40, 219.60,
     &   17.10, 228.60,  18.60, 237.70,  20.50, 246.90,  22.40, 256.40,
     &   24.00, 266.10,  25.40, 275.90,  26.70, 285.80,  28.10, 295.60,
     &   29.60, 305.60,  31.30, 315.90,  33.20, 326.60,  35.00, 338.10,
     &   36.20, 350.70,  36.10,   4.10,  34.20,  17.40,  30.40,  29.70,
     &   24.90,  40.40,  18.20,  49.60,  11.00,  57.80,   4.50,  65.60/
!  column   41 data, 36 LAT,LNG pairs
	DATA (CGMC(I, 41),I=1,72)/
     &   10.40,  74.00,  12.00,  82.70,  14.30,  91.70,  16.50, 101.20,
     &   18.40, 111.10,  19.50, 121.30,  19.90, 131.60,  19.60, 141.80,
     &   19.10, 151.70,  18.70, 161.40,  18.70, 171.10,  18.70, 181.00,
     &   18.70, 190.90,  18.60, 200.70,  18.30, 210.30,  18.40, 219.50,
     &   19.10, 228.50,  20.50, 237.50,  22.30, 246.60,  24.10, 256.00,
     &   25.80, 265.60,  27.20, 275.40,  28.60, 285.20,  30.10, 295.10,
     &   31.60, 305.10,  33.40, 315.50,  35.30, 326.30,  37.10, 338.10,
     &   38.20, 350.90,  38.00,   4.40,  36.10,  17.80,  32.50,  30.10,
     &   27.30,  40.80,  21.20,  49.90,  15.30,  58.10,  11.20,  65.90/
!  column   42 data, 36 LAT,LNG pairs
	DATA (CGMC(I, 42),I=1,72)/
     &   14.70,  74.30,  15.70,  82.80,  17.40,  91.90,  19.20, 101.30,
     &   20.70, 111.10,  21.80, 121.30,  22.10, 131.70,  21.80, 141.80,
     &   21.30, 151.70,  21.00, 161.50,  20.90, 171.20,  20.90, 181.10,
     &   20.90, 191.00,  20.60, 200.80,  20.40, 210.40,  20.40, 219.50,
     &   21.00, 228.40,  22.30, 237.30,  24.10, 246.30,  25.90, 255.60,
     &   27.60, 265.20,  29.10, 274.80,  30.60, 284.60,  32.10, 294.50,
     &   33.70, 304.60,  35.50, 315.00,  37.50, 326.10,  39.20, 338.00,
     &   40.20, 351.00,  40.00,   4.70,  38.10,  18.10,  34.50,  30.40,
     &   29.60,  41.10,  24.10,  50.30,  19.00,  58.40,  15.60,  66.20/
!  column   43 data, 36 LAT,LNG pairs
	DATA (CGMC(I, 43),I=1,72)/
     &   18.40,  74.60,  19.00,  83.10,  20.30,  92.00,  21.70, 101.40,
     &   23.10, 111.20,  24.00, 121.40,  24.20, 131.70,  24.00, 141.80,
     &   23.50, 151.70,  23.20, 161.50,  23.10, 171.30,  23.10, 181.10,
     &   23.00, 191.10,  22.70, 200.90,  22.40, 210.40,  22.40, 219.50,
     &   23.00, 228.30,  24.20, 237.10,  25.90, 246.00,  27.60, 255.20,
     &   29.30, 264.70,  30.90, 274.30,  32.50, 284.00,  34.10, 293.90,
     &   35.80, 304.10,  37.70, 314.60,  39.60, 325.80,  41.30, 338.00,
     &   42.20, 351.20,  41.90,   5.00,  40.00,  18.50,  36.60,  30.80,
     &   32.00,  41.50,  26.90,  50.60,  22.40,  58.70,  19.40,  66.60/
!  column   44 data, 36 LAT,LNG pairs
	DATA (CGMC(I, 44),I=1,72)/
     &   21.80,  74.90,  22.10,  83.30,  23.10,  92.20,  24.30, 101.50,
     &   25.40, 111.30,  26.20, 121.40,  26.40, 131.70,  26.10, 141.80,
     &   25.70, 151.80,  25.40, 161.50,  25.30, 171.30,  25.20, 181.20,
     &   25.10, 191.20,  24.80, 201.00,  24.40, 210.50,  24.40, 219.50,
     &   24.90, 228.20,  26.10, 236.90,  27.60, 245.70,  29.40, 254.80,
     &   31.10, 264.10,  32.80, 273.70,  34.40, 283.40,  36.10, 293.30,
     &   37.90, 303.50,  39.80, 314.20,  41.70, 325.50,  43.40, 337.90,
     &   44.20, 351.30,  43.80,   5.30,  41.90,  18.80,  38.60,  31.10,
     &   34.30,  41.80,  29.70,  50.90,  25.50,  59.10,  22.80,  66.90/
!  column   45 data, 36 LAT,LNG pairs
	DATA (CGMC(I, 45),I=1,72)/
     &   24.90,  75.20,  25.00,  83.60,  25.70,  92.40,  26.70, 101.70,
     &   27.70, 111.40,  28.30, 121.50,  28.50, 131.70,  28.30, 141.80,
     &   27.90, 151.80,  27.60, 161.60,  27.50, 171.40,  27.40, 181.30,
     &   27.20, 191.30,  26.80, 201.10,  26.50, 210.50,  26.40, 219.50,
     &   26.80, 228.70,  27.90, 236.70,  29.40, 245.40,  31.20, 254.40,
     &   32.90, 263.60,  34.60, 273.10,  36.30, 282.80,  38.10, 292.70,
     &   40.00, 302.90,  41.90, 313.70,  43.90, 325.30,  45.40, 337.90,
     &   46.20, 351.50,  45.70,   5.60,  43.90,  19.20,  40.70,  31.50,
     &   36.70,  42.10,  32.30,  51.30,  28.60,  59.40,  26.00,  67.20/
!  column   46 data, 36 LAT,LNG pairs
	DATA (CGMC(I, 46),I=1,72)/
     &   27.90,  75.60,  27.80,  83.90,  28.40,  92.60,  29.20, 101.90,
     &   30.00, 111.50,  30.50, 121.50,  30.60, 131.70,  30.40, 141.80,
     &   30.10, 151.80,  29.80, 161.60,  29.70, 171.50,  29.50, 181.40,
     &   29.30, 191.40,  28.90, 201.20,  28.50, 210.60,  28.40, 219.50,
     &   28.80, 228.10,  29.80, 236.50,  31.30, 245.10,  32.90, 254.00,
     &   34.70, 263.10,  36.40, 272.50,  38.20, 282.10,  40.10, 292.00,
     &   42.00, 302.30,  44.00, 313.20,  46.00, 325.00,  47.50, 337.80,
     &   48.20, 351.70,  47.70,   5.90,  45.80,  19.50,  42.80,  31.80,
     &   39.00,  42.50,  35.00,  51.60,  31.50,  59.80,  29.10,  67.60/
!  column   47 data, 36 LAT,LNG pairs
	DATA (CGMC(I, 47),I=1,72)/
     &   30.80,  75.90,  30.60,  84.20,  30.90,  92.90,  31.50, 102.10,
     &   32.20, 111.70,  32.60, 121.60,  32.80, 131.70,  32.60, 141.80,
     &   32.30, 151.80,  32.00, 161.60,  31.80, 171.50,  31.70, 181.50,
     &   31.40, 191.50,  31.00, 201.30,  30.60, 210.60,  30.40, 219.50,
     &   30.80, 228.00,  31.70, 236.30,  33.10, 244.80,  34.70, 253.50,
     &   36.50, 262.50,  38.30, 271.80,  40.10, 281.40,  42.10, 291.30,
     &   44.10, 301.70,  46.10, 312.70,  48.10, 324.70,  49.50, 337.80,
     &   50.20, 351.80,  49.60,   6.20,  47.80,  19.90,  44.90,  32.20,
     &   41.30,  42.80,  37.50,  52.00,  34.30,  60.20,  32.00,  68.00/
!  column   48 data, 36 LAT,LNG pairs
	DATA (CGMC(I, 48),I=1,72)/
     &   33.60,  76.40,  33.20,  84.60,  33.40,  93.20,  33.90, 102.30,
     &   34.40, 111.80,  34.80, 121.70,  34.90, 131.80,  34.70, 141.80,
     &   34.40, 151.80,  34.20, 161.70,  34.00, 171.60,  33.80, 181.60,
     &   33.50, 191.60,  33.00, 201.00,  32.60, 210.70,  32.40, 219.50,
     &   32.80, 227.90,  33.60, 236.20,  34.90, 244.50,  36.50, 253.10,
     &   38.30, 262.00,  40.10, 271.20,  42.10, 280.70,  44.10, 290.60,
     &   46.20, 301.00,  48.20, 312.20,  50.20, 324.40,  51.60, 337.70,
     &   52.20, 352.00,  51.60,   6.50,  49.80,  20.30,  47.00,  32.60,
     &   43.50,  43.20,  40.00,  52.40,  37.00,  60.60,  34.80,  68.50/
!  column   49 data, 36 LAT,LNG pairs
	DATA (CGMC(I, 49),I=1,72)/
     &   36.30,  76.80,  35.80,  85.00,  35.90,  93.60,  36.20, 102.60,
     &   36.60, 112.00,  36.90, 121.80,  37.00, 131.80,  36.80, 141.80,
     &   36.60, 151.80,  36.30, 161.70,  36.10, 171.70,  35.90, 181.70,
     &   35.50, 191.70,  35.10, 201.40,  34.70, 210.70,  34.50, 219.40,
     &   34.70, 227.80,  35.50, 236.00,  36.80, 244.20,  38.40, 252.70,
     &   40.10, 261.40,  42.00, 270.50,  44.00, 280.00,  46.10, 289.90,
     &   48.20, 300.40,  50.30, 311.70,  52.20, 324.10,  53.60, 337.70,
     &   54.10, 352.20,  53.50,   6.80,  51.70,  20.60,  49.00,  33.00,
     &   45.80,  43.60,  42.50,  52.80,  39.60,  61.10,  37.50,  69.00/
!  column   50 data, 36 LAT,LNG pairs
	DATA (CGMC(I, 50),I=1,72)/
     &   38.90,  77.30,  38.40,  85.50,  38.30,  94.00,  38.50, 102.90,
     &   38.80, 112.30,  39.00, 122.00,  39.00, 131.90,  38.90, 141.80,
     &   38.70, 151.80,  38.50, 161.70,  38.20, 171.80,  38.00, 181.80,
     &   37.60, 191.80,  37.20, 201.50,  36.70, 210.70,  36.50, 219.40,
     &   36.80, 227.60,  37.50, 235.70,  38.70, 243.90,  40.20, 252.20,
     &   42.00, 260.80,  43.90, 269.80,  45.90, 279.20,  48.10, 289.10,
     &   50.30, 299.70,  52.40, 311.10,  54.30, 323.70,  55.60, 337.60,
     &   56.10, 352.30,  55.50,   7.10,  53.70,  21.00,  51.10,  33.40,
     &   48.00,  44.00,  44.90,  53.30,  42.20,  61.60,  40.20,  69.50/
!  column   51 data, 36 LAT,LNG pairs
	DATA (CGMC(I, 51),I=1,72)/
     &   41.50,  77.90,  40.90,  86.00,  40.70,  94.40,  40.80, 103.30,
     &   40.90, 112.50,  41.10, 122.10,  41.10, 131.90,  41.00, 141.90,
     &   40.80, 151.80,  40.60, 161.80,  40.40, 171.80,  40.10, 181.90,
     &   39.70, 191.80,  39.20, 201.50,  38.80, 210.60,  38.60, 219.30,
     &   38.80, 227.50,  39.50, 235.50,  40.60, 243.50,  42.10, 251.70,
     &   43.90, 260.20,  45.80, 269.10,  47.90, 278.40,  50.10, 288.30,
     &   52.30, 298.90,  54.50, 310.50,  56.30, 323.40,  57.60, 337.50,
     &   58.10, 352.50,  57.40,   7.50,  55.70,  21.50,  53.20,  33.80,
     &   50.20,  44.50,  47.30,  53.80,  44.70,  62.10,  42.80,  70.00/
!  column   52 data, 36 LAT,LNG pairs
	DATA (CGMC(I, 52),I=1,72)/
     &   44.00,  78.50,  43.30,  86.50,  43.00,  94.90,  43.00, 103.70,
     &   43.10, 112.80,  43.20, 122.30,  43.20, 132.10,  43.10, 141.90,
     &   42.90, 151.90,  42.70, 161.80,  42.50, 171.90,  42.20, 181.90,
     &   41.80, 191.90,  41.30, 201.50,  40.80, 210.60,  40.60, 219.20,
     &   40.80, 227.30,  41.50, 235.20,  42.60, 243.10,  44.00, 251.60,
     &   45.80, 259.60,  47.70, 268.30,  49.80, 277.60,  52.10, 287.40,
     &   54.30, 298.10,  56.50, 309.90,  58.40, 323.00,  59.60, 337.40,
     &   60.00, 352.60,  59.30,   7.80,  57.60,  21.90,  55.20,  34.30,
     &   52.40,  45.00,  49.60,  54.30,  47.20,  62.70,  45.30,  70.60/
!  column   53 data, 36 LAT,LNG pairs
	DATA (CGMC(I, 53),I=1,72)/
     &   46.50,  79.10,  45.70,  87.10,  45.30,  95.40,  45.20, 104.10,
     &   45.20, 113.20,  45.20, 122.60,  45.20, 132.20,  45.10, 142.00,
     &   45.00, 151.90,  44.80, 161.90,  44.50, 171.90,  44.20, 182.00,
     &   43.80, 191.90,  43.30, 201.40,  42.90, 210.50,  42.70, 219.00,
     &   42.90, 227.10,  43.50, 234.90,  44.60, 242.70,  46.00, 250.70,
     &   47.70, 258.90,  49.60, 267.60,  51.80, 276.70,  54.10, 286.60,
     &   56.30, 297.30,  58.50, 309.20,  60.40, 322.50,  61.50, 337.20,
     &   62.00, 352.80,  61.30,   8.20,  59.60,  22.40,  57.30,  34.90,
     &   54.60,  45.60,  51.90,  54.90,  49.60,  63.30,  47.70,  71.30/
!  column   54 data, 36 LAT,LNG pairs
	DATA (CGMC(I, 54),I=1,72)/
     &   48.80,  79.80,  48.00,  87.80,  47.60,  96.00,  47.40, 104.60,
     &   47.30, 113.50,  47.30, 122.80,  47.30, 132.40,  47.20, 142.10,
     &   47.00, 152.00,  46.80, 162.00,  46.60, 172.00,  46.30, 182.00,
     &   45.90, 191.90,  45.40, 201.30,  45.00, 210.30,  44.80, 218.80,
     &   44.90, 226.80,  45.50, 234.50,  46.50, 242.20,  47.90, 250.10,
     &   49.60, 258.20,  51.60, 266.70,  53.80, 275.80,  56.00, 285.60,
     &   58.40, 296.40,  60.60, 308.50,  62.40, 322.00,  63.60, 337.10,
     &   63.90, 352.90,  63.20,   8.50,  61.60,  22.90,  59.30,  35.50,
     &   56.70,  46.20,  54.20,  55.60,  52.00,  64.00,  50.10,  72.00/
!  column   55 data, 36 LAT,LNG pairs
	DATA (CGMC(I, 55),I=1,72)/
     &   51.20,  80.60,  50.30,  88.50,  49.80,  96.70,  49.50, 105.10,
     &   49.40, 114.00,  49.30, 123.20,  49.30, 132.60,  49.20, 142.30,
     &   49.10, 152.10,  48.90, 162.00,  48.70, 172.00,  48.30, 182.00,
     &   47.90, 191.80,  47.50, 201.20,  47.10, 210.10,  46.90, 218.50,
     &   47.00, 226.40,  47.60, 234.10,  48.50, 241.70,  49.90, 249.40,
     &   51.60, 257.40,  53.60, 265.90,  55.70, 274.90,  58.00, 284.60,
     &   60.40, 295.50,  62.50, 307.70,  64.40, 321.50,  65.50, 336.80,
     &   65.80, 353.10,  65.10,   9.00,  63.50,  23.50,  61.30,  36.10,
     &   58.90,  46.90,  56.40,  56.40,  54.30,  64.80,  52.50,  72.80/
!  column   56 data, 36 LAT,LNG pairs
	DATA (CGMC(I, 56),I=1,72)/
     &   53.50,  81.40,  52.60,  89.30,  52.00,  97.40,  51.60, 105.80,
     &   51.50, 114.50,  51.40, 123.50,  51.30, 132.90,  51.20, 142.50,
     &   51.10, 152.20,  50.90, 162.10,  50.70, 172.10,  50.40, 182.00,
     &   50.00, 191.70,  49.50, 201.00,  49.20, 209.90,  49.00, 218.20,
     &   49.10, 226.00,  49.60, 233.60,  50.60, 241.10,  51.90, 248.70,
     &   53.60, 256.60,  55.50, 264.90,  57.70, 273.80,  60.00, 283.60,
     &   62.30, 294.40,  64.50, 306.80,  66.30, 320.90,  67.50, 336.60,
     &   67.70, 353.20,  67.00,   9.40,  65.40,  24.10,  63.30,  36.90,
     &   61.00,  47.80,  58.60,  57.20,  56.50,  65.70,  54.80,  73.60/
!  column   57 data, 36 LAT,LNG pairs
	DATA (CGMC(I, 57),I=1,72)/
     &   55.70,  82.40,  54.80,  90.20,  54.20,  98.20,  53.80, 106.40,
     &   53.50, 115.10,  53.40, 124.00,  53.30, 133.20,  53.20, 142.70,
     &   53.10, 152.40,  52.90, 162.20,  52.70, 172.10,  52.40, 181.90,
     &   52.00, 191.60,  51.60, 200.80,  51.20, 209.50,  51.10, 217.80,
     &   51.20, 225.60,  51.70, 233.10,  52.60, 240.50,  53.90, 248.00,
     &   55.60, 255.70,  57.50, 263.90,  59.70, 272.70,  62.00, 282.40,
     &   64.30, 293.30,  66.50, 305.80,  68.30, 320.20,  69.40, 336.30,
     &   69.60, 353.30,  68.90,   9.90,  67.40,  24.90,  65.30,  37.70,
     &   63.00,  48.70,  60.80,  58.10,  58.70,  66.60,  57.00,  74.60/
!  column   58 data, 36 LAT,LNG pairs
	DATA (CGMC(I, 58),I=1,72)/
     &   57.90,  83.40,  57.00,  91.10,  56.30,  99.00,  55.90, 107.20,
     &   55.60, 115.70,  55.40, 124.50,  55.30, 133.60,  55.20, 143.00,
     &   55.10, 152.60,  54.90, 162.30,  54.70, 172.10,  54.40, 181.90,
     &   54.00, 191.40,  53.60, 200.50,  53.30, 209.10,  53.20, 217.30,
     &   53.30, 225.00,  53.80, 232.40,  54.70, 239.70,  56.00, 247.10,
     &   57.60, 254.70,  59.50, 262.80,  61.70, 271.50,  63.90, 281.10,
     &   66.30, 292.00,  68.40, 304.60,  70.20, 319.30,  71.30, 335.90,
     &   71.50, 353.50,  70.80,  10.50,  69.30,  25.80,  67.30,  38.70,
     &   65.00,  49.70,  62.90,  59.20,  60.90,  67.70,  59.30,  75.70/
!  column   59 data, 36 LAT,LNG pairs
	DATA (CGMC(I, 59),I=1,72)/
     &   60.10,  84.50,  59.10,  92.00,  58.40, 100.00,  57.90, 108.10,
     &   57.60, 116.50,  57.40, 125.10,  57.30, 134.10,  57.20, 143.40,
     &   57.00, 152.80,  56.90, 162.50,  56.60, 172.10,  56.40, 181.80,
     &   56.00, 191.10,  55.70, 200.10,  55.40, 208.70,  55.30, 216.70,
     &   55.50, 224.30,  56.00, 231.70,  56.80, 238.90,  58.10, 246.10,
     &   59.60, 253.60,  61.50, 261.50,  63.60, 270.10,  65.90, 279.00,
     &   68.20, 290.60,  70.30, 303.30,  72.10, 318.30,  73.20, 335.40,
     &   73.40, 353.70,  72.60,  11.20,  71.20,  26.80,  69.20,  39.90,
     &   67.10,  51.00,  65.00,  60.50,  63.00,  68.90,  61.40,  76.90/
!  column   60 data, 36 LAT,LNG pairs
	DATA (CGMC(I, 60),I=1,72)/
     &   62.30,  85.90,  61.20,  93.40,  60.50, 101.20,  60.00, 109.10,
     &   59.60, 117.30,  59.40, 125.80,  59.20, 134.70,  59.10, 143.80,
     &   59.00, 153.10,  58.80, 162.60,  58.60, 172.20,  58.30, 181.60,
     &   58.00, 190.80,  57.70, 199.70,  57.50, 208.10,  57.40, 216.00,
     &   57.60, 223.60,  58.10, 230.80,  58.90, 237.90,  60.10, 245.00,
     &   61.70, 252.40,  63.50, 260.10,  65.60, 268.60,  67.80, 278.00,
     &   70.10, 288.90,  72.20, 301.80,  73.90, 317.10,  75.00, 334.80,
     &   75.20, 353.80,  74.50,  12.10,  73.00,  28.00,  71.10,  41.30,
     &   69.00,  52.40,  67.00,  61.90,  65.10,  70.40,  63.50,  78.20/
!  column   61 data, 36 LAT,LNG pairs
	DATA (CGMC(I, 61),I=1,72)/
     &   64.30,  87.40,  63.30,  94.80,  62.60, 102.40,  62.00, 110.20,
     &   61.60, 118.30,  61.30, 126.70,  61.20, 135.30,  61.00, 144.30,
     &   60.90, 153.50,  60.70, 162.80,  60.50, 172.10,  60.30, 181.40,
     &   60.00, 190.50,  59.70, 199.20,  59.50, 207.40,  59.50, 215.20,
     &   59.70, 222.70,  60.20, 229.80,  61.00, 236.70,  62.20, 243.70,
     &   63.70, 250.90,  65.50, 258.50,  67.50, 266.80,  69.70, 276.10,
     &   71.90, 286.90,  74.00, 299.90,  75.80, 315.60,  76.90, 334.10,
     &   77.10, 354.10,  76.30,  13.10,  74.90,  29.50,  73.00,  43.00,
     &   71.00,  54.10,  69.00,  63.60,  67.20,  72.00,  65.60,  79.80/
!  column   62 data, 36 LAT,LNG pairs
	DATA (CGMC(I, 62),I=1,72)/
     &   66.40,  89.10,  65.40,  96.50,  64.60, 103.90,  64.00, 111.60,
     &   63.60, 119.50,  63.30, 127.60,  63.10, 136.10,  62.90, 144.90,
     &   62.80, 153.90,  62.60, 163.00,  62.40, 172.10,  62.20, 181.20,
     &   61.90, 190.00,  61.70, 198.50,  61.60, 206.60,  61.60, 214.30,
     &   61.80, 221.60,  62.30, 228.60,  63.20, 235.40,  64.30, 242.30,
     &   65.80, 249.30,  67.50, 256.70,  69.50, 264.80,  71.60, 273.90,
     &   73.80, 284.60,  75.90, 297.50,  77.60, 313.60,  78.70, 333.10,
     &   78.90, 354.30,  78.10,  14.50,  76.70,  31.40,  74.80,  45.10,
     &   72.90,  56.20,  71.00,  65.60,  69.20,  73.90,  67.70,  81.70/
!  column   63 data, 36 LAT,LNG pairs
	DATA (CGMC(I, 63),I=1,72)/
     &   68.40,  91.10,  67.40,  98.30,  66.60, 105.60,  66.00, 113.10,
     &   65.50, 120.80,  65.20, 128.80,  65.00, 137.10,  64.80, 145.60,
     &   64.60, 154.30,  64.50, 163.20,  64.30, 172.10,  64.10, 180.90,
     &   63.90, 189.50,  63.70, 197.80,  63.60, 205.70,  63.70, 213.20,
     &   63.90, 220.30,  64.50, 227.20,  65.30, 233.80,  66.40, 240.50,
     &   67.80, 247.30,  69.50, 254.50,  71.40, 262.30,  73.40, 271.20,
     &   75.60, 281.70,  77.60, 294.60,  79.40, 311.10,  80.50, 331.70,
     &   80.70, 354.60,  79.90,  16.20,  78.50,  33.90,  76.70,  47.80,
     &   74.70,  58.80,  72.90,  68.10,  71.20,  76.30,  69.70,  83.80/
!  column   64 data, 36 LAT,LNG pairs
	DATA (CGMC(I, 64),I=1,72)/
     &   70.40,  93.50,  69.40, 100.50,  68.60, 107.60,  68.00, 114.90,
     &   67.50, 122.40,  67.10, 130.10,  66.90, 138.10,  66.70, 146.40,
     &   66.50, 154.90,  66.30, 163.50,  66.10, 172.10,  66.00, 180.60,
     &   65.80, 188.90,  65.70, 196.90,  65.60, 204.60,  65.70, 211.90,
     &   66.00, 218.80,  66.60, 225.50,  67.40, 232.00,  68.50, 238.40,
     &   69.80, 245.00,  71.40, 251.90,  73.20, 259.40,  75.20, 267.90,
     &   77.30, 278.00,  79.30, 290.80,  81.10, 307.60,  82.20, 329.70,
     &   82.40, 355.10,  81.70,  18.70,  80.20,  37.30,  78.40,  51.20,
     &   76.50,  62.10,  74.70,  71.10,  73.10,  79.10,  71.70,  86.40/
!  column   65 data, 36 LAT,LNG pairs
	DATA (CGMC(I, 65),I=1,72)/
     &   72.40,  96.40,  71.40, 103.20,  70.50, 110.00,  69.90, 117.00,
     &   69.40, 124.20,  69.00, 131.70,  68.70, 139.40,  68.50, 147.40,
     &   68.30, 155.50,  68.10, 163.80,  68.00, 172.00,  67.80, 180.20,
     &   67.70, 188.20,  67.60, 195.90,  67.60, 203.30,  67.80, 210.30,
     &   68.10, 217.00,  68.70, 223.40,  69.40, 229.70,  70.50, 235.90,
     &   71.80, 242.20,  73.30, 248.80,  75.10, 255.80,  77.00, 263.80,
     &   79.00, 273.40,  81.00, 285.70,  82.70, 302.60,  84.00, 326.50,
     &   84.20, 355.80,  83.40,  22.50,  81.90,  42.00,  80.10,  55.80,
     &   78.30,  66.20,  76.60,  76.00,  75.00,  82.60,  73.60,  89.60/
!  column   66 data, 36 LAT,LNG pairs
	DATA (CGMC(I, 66),I=1,72)/
     &   74.20, 100.00,  73.20, 106.40,  72.40, 112.90,  71.80, 119.60,
     &   71.30, 126.50,  70.90, 133.60,  70.60, 141.00,  70.30, 148.60,
     &   70.10, 156.30,  69.90, 164.10,  69.80, 172.00,  69.70, 179.70,
     &   69.60, 187.30,  69.50, 194.60,  69.60, 201.70,  69.80, 208.40,
     &   70.20, 214.80,  70.70, 221.00,  71.50, 227.00,  72.50, 232.80,
     &   73.70, 238.70,  75.20, 244.80,  76.80, 251.30,  78.60, 258.60,
     &   80.50, 267.20,  82.50, 278.50,  84.30, 294.90,  85.60, 321.00,
     &   86.00, 357.20,  85.10,  28.80,  83.50,  49.00,  81.70,  62.20,
     &   79.90,  72.00,  78.30,  79.90,  76.80,  87.00,  75.40,  93.60/
!  column   67 data, 36 LAT,LNG pairs
	DATA (CGMC(I, 67),I=1,72)/
     &   76.00, 104.50,  75.10, 110.40,  74.30, 116.50,  73.60, 122.80,
     &   73.10, 129.20,  72.70, 135.90,  72.40, 142.80,  72.10, 150.00,
     &   71.90, 157.20,  71.70, 164.50,  71.60, 171.90,  71.50, 179.20,
     &   71.40, 186.30,  71.50, 193.20,  71.60, 199.80,  71.80, 206.10,
     &   72.20, 212.20,  72.70, 218.00,  73.50, 223.60,  74.40, 229.00,
     &   75.60, 234.40,  76.90, 239.90,  78.40, 245.60,  80.10, 251.70,
     &   81.90, 258.90,  83.80, 268.10,  85.60, 282.00,  87.10, 308.90,
     &   87.60, 359.80,  86.50,  40.00,  84.90,  60.20,  83.20,  71.40,
     &   81.50,  79.60,  79.90,  86.50,  78.40,  92.70,  77.20,  98.60/
!  column   68 data, 36 LAT,LNG pairs
	DATA (CGMC(I, 68),I=1,72)/
     &   77.70, 110.30,  76.80, 115.50,  76.10, 121.00,  75.40, 126.70,
     &   74.90, 132.60,  74.50, 138.80,  74.10, 145.10,  73.80, 151.70,
     &   73.60, 158.30,  73.50, 165.10,  73.40, 171.80,  73.30, 178.50,
     &   73.30, 185.00,  73.30, 191.30,  73.50, 197.40,  73.70, 203.30,
     &   74.10, 208.90,  74.70, 214.20,  75.40, 219.30,  76.30, 224.10,
     &   77.30, 228.90,  78.50, 233.50,  79.90, 238.10,  81.50, 242.70,
     &   83.10, 247.60,  84.80, 255.00,  86.50, 260.10,  88.40, 275.10,
     &   89.40,  24.00,  88.00,  67.10,  86.00,  78.60,  84.40,  85.00,
     &   82.80,  90.30,  81.30,  95.30,  80.00, 100.20,  78.80, 105.20/
!  column   69 data, 36 LAT,LNG pairs
	DATA (CGMC(I, 69),I=1,72)/
     &   79.30, 117.80,  78.50, 122.10,  77.70, 126.70,  77.10, 131.70,
     &   76.60, 136.90,  76.20, 142.30,  75.80, 148.00,  75.60, 153.80,
     &   75.40, 159.70,  75.20, 165.70,  75.10, 171.70,  75.10, 177.60,
     &   75.10, 183.40,  75.20, 189.10,  75.40, 194.50,  75.60, 199.70,
     &   76.00, 204.70,  76.50, 209.40,  77.20, 213.80,  78.00, 217.90,
     &   78.90, 221.70,  80.00, 225.20,  81.20, 228.30,  82.50, 230.80,
     &   83.90, 232.60,  85.40, 232.80,  86.80, 229.60,  88.20, 214.60,
     &   88.80, 157.40,  87.90, 114.50,  86.50, 105.80,  85.10, 103.90,
     &   83.70, 104.90,  82.50, 107.20,  81.30, 110.20,  80.20, 113.80/
!  column   70 data, 36 LAT,LNG pairs
	DATA (CGMC(I, 70),I=1,72)/
     &   80.60, 127.50,  79.90, 130.60,  79.30, 134.20,  78.70, 138.10,
     &   78.20, 142.30,  77.80, 146.80,  77.50, 151.50,  77.30, 156.40,
     &   77.10, 161.40,  76.90, 166.50,  76.80, 171.50,  76.80, 176.50,
     &   76.90, 181.50,  77.00, 186.20,  77.20, 190.80,  77.40, 195.20,
     &   77.80, 199.30,  78.30, 203.10,  78.90, 206.60,  79.50, 209.70,
     &   80.30, 212.30,  81.20, 214.40,  82.10, 215.80,  83.10, 216.10,
     &   84.20, 214.80,  85.20, 210.90,  86.20, 202.80,  86.90, 187.80,
     &   87.10, 166.20,  86.80, 146.60,  86.10, 133.10,  85.10, 126.10,
     &   84.20, 123.10,  83.20, 122.40,  82.30, 123.20,  81.40, 125.00/
!  column   71 data, 36 LAT,LNG pairs
	DATA (CGMC(I, 71),I=1,72)/
     &   81.60, 140.00,  81.00, 141.60,  80.50, 143.80,  80.10, 146.50,
     &   79.70, 149.40,  79.40, 152.70,  79.10, 156.20,  78.90, 159.80,
     &   78.70, 163.60,  78.60, 167.40,  78.60, 171.30,  78.60, 175.10,
     &   78.60, 178.90,  78.70, 182.50,  78.90, 186.00,  79.10, 189.20,
     &   79.50, 192.20,  79.80, 194.90,  80.30, 197.20,  80.80, 199.10,
     &   81.40, 200.40,  82.00, 201.10,  82.60, 200.90,  83.30, 199.60,
     &   83.90, 197.00,  84.50, 192.70,  85.00, 186.40,  85.90, 178.10,
     &   85.40, 169.00,  85.30, 159.80,  85.00, 152.00,  84.50, 146.00,
     &   84.00, 142.10,  83.40, 139.80,  82.80, 138.90,  82.10, 139.00/
!  column   72 data, 36 LAT,LNG pairs
	DATA (CGMC(I, 72),I=1,72)/
     &   82.10, 154.90,  81.80, 155.30,  81.50, 156.10,  81.20, 157.30,
     &   81.00, 158.70,  80.80, 160.40,  80.60, 162.30,  80.50, 164.40,
     &   80.40, 166.60,  80.30, 168.80,  80.30, 171.00,  80.30, 173.30,
     &   80.30, 175.50,  80.40, 177.60,  80.50, 179.50,  80.70, 181.30,
     &   80.90, 182.90,  81.10, 184.20,  81.40, 185.20,  81.70, 185.80,
     &   82.00, 186.10,  82.30, 185.90,  82.60, 185.10,  82.90, 183.80,
     &   83.10, 182.00,  83.40, 179.60,  83.60, 176.60,  83.70, 173.50,
     &   83.70, 170.00,  83.70, 166.60,  83.60, 163.40,  83.40, 160.60,
     &   83.20, 158.30,  82.90, 156.60,  82.70, 155.50,  82.40, 154.90/
!  column   73 data, 36 LAT,LNG pairs
	DATA (CGMC(I, 73),I=1,72)/
     &  -75.80, 129.40, -75.90, 128.20, -75.90, 126.80, -75.80, 125.10,
     &  -75.70, 123.80, -75.50, 122.70, -75.30, 121.80, -75.00, 120.90,
     &  -74.70, 120.30, -74.40, 120.00, -74.00, 119.60, -73.70, 119.70,
     &  -73.30, 119.90, -73.00, 120.30, -72.80, 120.70, -72.50, 121.70,
     &  -72.30, 122.50, -72.20, 123.20, -72.20, 124.20, -72.10, 125.00,
     &  -72.10, 126.80, -72.10, 127.90, -72.20, 128.80, -72.40, 129.80,
     &  -72.60, 130.90, -72.80, 131.50, -73.10, 132.20, -73.40, 132.80,
     &  -73.70, 133.30, -74.10, 133.50, -74.40, 133.60, -74.70, 133.20,
     &  -75.00, 132.90, -75.30, 132.30, -75.50, 131.50, -75.70, 130.50/
!  column   74 data, 36 LAT,LNG pairs
	DATA (CGMC(I, 74),I=1,72)/
     &  -77.60, 132.50, -77.80, 129.50, -77.80, 126.20, -77.70, 122.90,
     &  -77.40, 120.00, -77.00, 117.40, -76.40, 115.30, -75.80, 113.90,
     &  -75.20, 112.70, -74.50, 112.20, -73.80, 112.20, -73.10, 112.60,
     &  -72.50, 113.30, -71.90, 114.40, -71.40, 115.70, -70.90, 117.20,
     &  -70.60, 119.00, -70.30, 120.80, -70.20, 122.70, -70.10, 124.70,
     &  -70.10, 126.70, -70.30, 128.70, -70.50, 130.70, -70.70, 132.60,
     &  -71.10, 134.50, -71.50, 136.20, -72.00, 137.70, -72.60, 139.00,
     &  -73.20, 140.20, -73.80, 140.90, -74.50, 141.20, -75.20, 141.10,
     &  -75.80, 140.40, -76.40, 139.20, -76.90, 137.50, -77.30, 135.30/
!  column   75 data, 36 LAT,LNG pairs
	DATA (CGMC(I, 75),I=1,72)/
     &  -79.50, 137.00, -79.80, 131.70, -79.80, 125.80, -79.60, 120.00,
     &  -79.10, 114.80, -78.30, 110.70, -77.40, 107.80, -76.40, 105.90,
     &  -75.40, 104.90, -74.30, 104.70, -73.30, 105.10, -72.30, 106.20,
     &  -71.40, 107.60, -70.60, 109.40, -69.80, 111.30, -69.30, 113.60,
     &  -68.80, 116.00, -68.40, 118.60, -68.20, 121.30, -68.10, 124.00,
     &  -68.20, 126.80, -68.40, 129.60, -68.60, 132.30, -69.00, 135.00,
     &  -69.50, 137.60, -70.20, 140.10, -70.90, 142.40, -71.70, 144.50,
     &  -72.50, 146.20, -73.40, 147.60, -74.40, 148.50, -75.40, 148.90,
     &  -76.30, 148.50, -77.30, 147.30, -78.20, 145.10, -78.90, 141.70/
!  column   76 data, 36 LAT,LNG pairs
	DATA (CGMC(I, 76),I=1,72)/
     &  -81.10, 144.10, -81.70, 135.50, -81.80, 125.20, -81.40, 115.70,
     &  -80.60, 108.00, -79.50, 102.70, -78.20,  99.30, -76.80,  97.50,
     &  -75.40,  97.00, -74.00,  97.40, -72.60,  98.60, -71.30, 100.30,
     &  -70.20, 102.50, -69.20, 105.00, -68.30, 107.70, -67.60, 110.60,
     &  -67.10, 113.70, -66.60, 116.90, -66.40, 120.10, -66.30, 123.50,
     &  -66.40, 126.80, -66.60, 130.20, -66.90, 133.60, -67.40, 137.00,
     &  -68.00, 140.20, -68.80, 143.40, -69.60, 146.50, -70.60, 149.30,
     &  -71.70, 151.90, -72.80, 154.10, -74.00, 155.80, -75.30, 156.80,
     &  -76.60, 157.10, -77.90, 156.20, -79.10, 153.80, -80.20, 149.70/
!  column   77 data, 36 LAT,LNG pairs
	DATA (CGMC(I, 77),I=1,72)/
     &  -82.60, 151.90, -83.50, 140.20, -83.80, 124.10, -83.10, 109.30,
     &  -82.00,  98.70, -80.40,  92.30, -78.70,  89.50, -76.90,  88.60,
     &  -75.10,  89.10, -73.40,  90.60, -71.70,  92.50, -70.20,  95.00,
     &  -68.90,  97.80, -67.70, 101.00, -66.70, 104.40, -65.80, 107.90,
     &  -65.20, 111.50, -64.70, 115.20, -64.40, 119.10, -64.40, 123.00,
     &  -64.50, 126.90, -64.70, 130.80, -65.10, 134.80, -65.70, 138.70,
     &  -66.40, 142.60, -67.30, 146.40, -68.30, 150.10, -69.40, 153.70,
     &  -70.60, 157.00, -72.00, 160.00, -73.50, 162.60, -75.00, 164.70,
     &  -76.60, 166.00, -78.20, 166.00, -79.80, 164.50, -81.40, 160.40/
!  column   78 data, 36 LAT,LNG pairs
	DATA (CGMC(I, 78),I=1,72)/
     &  -84.00, 165.00, -85.30, 148.00, -85.80, 121.70, -84.70,  98.00,
     &  -83.00,  85.40, -81.00,  80.40, -78.80,  79.00, -76.60,  79.60,
     &  -74.50,  81.40, -72.50,  83.90, -70.70,  86.90, -68.90,  90.20,
     &  -67.50,  93.90, -66.10,  97.60, -64.90, 101.50, -64.00, 105.50,
     &  -63.30, 109.70, -62.80, 113.90, -62.50, 118.20, -62.40, 122.50,
     &  -62.60, 127.00, -62.90, 131.40, -63.40, 135.80, -64.00, 140.20,
     &  -64.80, 144.60, -65.80, 149.00, -66.90, 153.30, -68.10, 157.40,
     &  -69.50, 161.40, -71.10, 165.20, -72.70, 168.80, -74.40, 171.80,
     &  -76.30, 174.20, -78.20, 175.80, -80.10, 175.80, -82.10, 173.40/
!  column   79 data, 36 LAT,LNG pairs
	DATA (CGMC(I, 79),I=1,72)/
     &  -84.70, 185.00, -87.20, 168.00, -87.70, 111.00, -86.00,  78.20,
     &  -83.60,  68.30, -81.10,  67.00, -78.60,  68.40, -76.10,  70.80,
     &  -73.70,  74.20, -71.50,  77.80, -69.40,  81.80, -67.50,  85.80,
     &  -65.80,  90.10, -64.40,  94.40, -63.20,  98.90, -62.20, 103.40,
     &  -61.40, 108.00, -60.90, 112.70, -60.60, 117.40, -60.60, 122.20,
     &  -60.70, 126.90, -61.10, 131.70, -61.60, 136.70, -62.30, 141.60,
     &  -63.20, 146.50, -64.20, 151.30, -65.40, 156.10, -66.80, 160.80,
     &  -68.30, 165.40, -70.00, 169.90, -71.70, 174.30, -73.60, 178.40,
     &  -75.70, 182.10, -77.80, 185.20, -80.00, 187.70, -82.40, 187.80/
!  column   80 data, 36 LAT,LNG pairs
	DATA (CGMC(I, 80),I=1,72)/
     &  -84.80, 206.10, -87.80, 208.00, -89.30,  77.00, -86.60,  47.00,
     &  -83.60,  49.50, -80.70,  53.50, -77.90,  58.00, -75.20,  62.80,
     &  -72.60,  67.50, -70.20,  72.30, -68.00,  77.20, -66.00,  81.90,
     &  -64.20,  86.80, -62.60,  91.70, -61.40,  96.60, -60.40, 101.60,
     &  -59.60, 106.60, -59.10, 111.60, -58.80, 116.70, -58.70, 121.90,
     &  -58.90, 127.10, -59.30, 132.30, -59.80, 137.50, -60.60, 142.80,
     &  -61.50, 148.10, -62.60, 153.30, -63.90, 158.60, -65.40, 163.90,
     &  -67.00, 169.10, -68.70, 174.30, -70.60, 179.30, -72.70, 184.30,
     &  -74.80, 189.20, -77.10, 194.00, -79.60, 198.60, -82.10, 203.00/
!  column   81 data, 36 LAT,LNG pairs
	DATA (CGMC(I, 81),I=1,72)/
     &  -84.20, 227.20, -87.10, 249.10, -88.20, 323.10, -85.80,  16.30,
     &  -82.80,  32.40, -79.80,  41.80, -76.90,  49.00, -74.10,  55.50,
     &  -71.30,  61.50, -68.80,  67.20, -66.50,  72.90, -64.40,  78.50,
     &  -62.50,  83.90, -60.90,  89.20, -59.60,  94.60, -58.50,  99.90,
     &  -57.70, 105.30, -57.20, 110.70, -56.90, 116.10, -56.90, 121.60,
     &  -57.10, 127.10, -57.50, 132.70, -58.10, 138.30, -58.90, 143.90,
     &  -59.90, 149.50, -61.10, 155.10, -62.40, 160.80, -63.90, 166.60,
     &  -65.60, 172.30, -67.40, 178.10, -69.40, 183.90, -71.50, 189.60,
     &  -73.80, 195.60, -76.20, 201.80, -78.70, 208.20, -81.40, 215.90/
!  column   82 data, 36 LAT,LNG pairs
	DATA (CGMC(I, 82),I=1,72)/
     &  -83.10, 241.40, -85.50, 269.50, -86.10, 320.60, -84.30, 359.30,
     &  -81.50,  19.50, -78.60,  32.00, -75.60,  41.20, -72.70,  49.10,
     &  -69.90,  56.20, -67.30,  62.80, -64.90,  69.20, -62.70,  75.30,
     &  -60.80,  81.20, -59.10,  87.00, -57.70,  92.70, -56.70,  98.40,
     &  -55.90, 104.10, -55.30, 109.80, -55.10, 115.60, -55.00, 121.30,
     &  -55.20, 127.10, -55.70, 133.00, -56.30, 138.90, -57.10, 144.90,
     &  -58.20, 150.80, -59.40, 156.90, -60.80, 163.00, -62.40, 169.10,
     &  -64.10, 175.30, -66.00, 181.50, -68.00, 187.80, -70.20, 194.40,
     &  -72.60, 201.10, -75.00, 208.20, -77.70, 216.40, -80.40, 226.60/
!  column   83 data, 36 LAT,LNG pairs
	DATA (CGMC(I, 83),I=1,72)/
     &  -81.70, 251.80, -83.70, 278.90, -84.10, 317.50, -82.50, 349.70,
     &  -79.90,  10.30, -77.00,  24.10, -74.00,  34.60, -71.00,  43.40,
     &  -68.20,  51.30, -65.50,  58.70, -63.10,  65.70, -60.90,  72.40,
     &  -58.90,  78.70, -57.20,  84.80, -55.80,  90.90, -54.70,  96.90,
     &  -54.00, 103.00, -53.50, 109.00, -53.20, 115.10, -53.20, 121.10,
     &  -53.40, 127.20, -53.80, 133.40, -54.50, 139.60, -55.40, 145.80,
     &  -56.50, 152.00, -57.70, 158.40, -59.20, 164.80, -60.90, 171.30,
     &  -62.70, 177.80, -64.60, 184.50, -66.70, 191.40, -68.80, 198.50,
     &  -71.20, 205.80, -73.70, 213.90, -76.40, 223.20, -79.10, 235.00/
!  column   84 data, 36 LAT,LNG pairs
	DATA (CGMC(I, 84),I=1,72)/
     &  -80.00, 259.10, -81.80, 284.20, -82.10, 316.20, -80.60, 344.00,
     &  -78.10,   3.90, -75.10,  18.00, -72.10,  29.10, -69.10,  38.50,
     &  -66.30,  47.00, -63.70,  55.00, -61.30,  62.60, -59.00,  69.70,
     &  -57.00,  76.40, -55.30,  83.00, -53.90,  89.40, -52.90,  95.70,
     &  -52.10, 102.00, -51.60, 108.30, -51.40, 114.60, -51.40, 120.90,
     &  -51.60, 127.30, -52.10, 133.70, -52.70, 140.10, -53.60, 146.60,
     &  -54.80, 153.20, -56.10, 159.70, -57.60, 166.50, -59.20, 173.30,
     &  -61.10, 180.20, -63.10, 187.20, -65.20, 194.50, -67.40, 202.00,
     &  -69.80, 210.00, -72.30, 218.60, -74.90, 228.80, -77.60, 241.30/
!  column   85 data, 36 LAT,LNG pairs
	DATA (CGMC(I, 85),I=1,72)/
     &  -78.40, 263.80, -79.90, 287.60, -80.00, 315.60, -78.50, 340.40,
     &  -76.00, 359.10, -73.00,  13.10, -70.10,  24.50, -67.10,  34.20,
     &  -64.30,  43.20, -61.80,  51.70, -59.30,  59.60, -57.10,  67.10,
     &  -55.10,  74.30, -53.50,  81.30, -52.10,  87.90, -51.00,  94.50,
     &  -50.20, 101.00, -49.70, 107.60, -49.50, 114.10, -49.60, 120.70,
     &  -49.80, 127.30, -50.30, 134.00, -51.00, 140.60, -51.90, 147.30,
     &  -53.10, 154.20, -54.40, 161.10, -55.90, 168.00, -57.70, 175.10,
     &  -59.50, 182.30, -61.50, 189.70, -63.60, 197.30, -65.90, 205.20,
     &  -68.30, 213.60, -70.80, 222.80, -73.40, 233.30, -76.00, 246.30/
!  column   86 data, 36 LAT,LNG pairs
	DATA (CGMC(I, 86),I=1,72)/
     &  -76.60, 267.60, -78.00, 290.00, -77.90, 315.30, -76.30, 337.80,
     &  -73.70, 355.50, -70.80,   9.30, -67.80,  20.60, -64.90,  30.40,
     &  -62.20,  39.90, -59.60,  48.60, -57.20,  56.80, -55.10,  64.80,
     &  -53.20,  72.30, -51.50,  79.70, -50.10,  86.50, -49.10,  93.40,
     &  -48.30, 100.20, -47.80, 106.90, -47.70, 113.70, -47.70, 120.50,
     &  -48.00, 127.40, -48.50, 134.20, -49.20, 141.10, -50.20, 148.10,
     &  -51.40, 155.10, -52.70, 162.20, -54.30, 169.40, -56.00, 176.80,
     &  -57.90, 184.30, -59.90, 192.00, -62.00, 199.90, -64.30, 208.10,
     &  -66.60, 216.80, -69.20, 226.20, -71.70, 237.10, -74.30, 250.40/
!  column   87 data, 36 LAT,LNG pairs
	DATA (CGMC(I, 87),I=1,72)/
     &  -74.80, 270.60, -76.00, 291.80, -75.80, 315.10, -74.00, 336.20,
     &  -71.40, 352.90, -68.40,   6.10, -65.30,  17.20, -62.50,  27.20,
     &  -59.80,  36.60, -57.40,  45.60, -55.10,  54.30, -53.00,  62.60,
     &  -51.20,  70.50, -49.50,  78.00, -48.20,  85.20, -47.20,  92.30,
     &  -46.40,  99.40, -46.00, 106.30, -45.80, 113.30, -45.90, 120.30,
     &  -46.20, 127.40, -46.70, 134.50, -47.50, 141.60, -48.40, 148.80,
     &  -49.60, 156.00, -51.00, 163.30, -52.60, 170.70, -54.30, 178.30,
     &  -56.20, 186.00, -58.20, 194.00, -60.40, 202.20, -62.60, 210.60,
     &  -65.00, 219.60, -67.50, 229.30, -70.10, 240.20, -72.60, 253.70/
!  column   88 data, 36 LAT,LNG pairs
	DATA (CGMC(I, 88),I=1,72)/
     &  -72.90, 272.80, -74.00, 293.40, -73.60, 315.20, -71.70, 334.90,
     &  -69.00, 350.80, -65.80,   3.50, -62.70,  14.40, -59.90,  24.40,
     &  -57.40,  33.80, -55.10,  43.00, -52.90,  51.90, -50.90,  60.50,
     &  -49.10,  68.80, -47.50,  76.50, -46.20,  84.00, -45.20,  91.30,
     &  -44.70,  98.60, -44.10, 105.70, -44.00, 112.90, -44.10, 120.20,
     &  -44.40, 127.50, -44.90, 134.80, -45.70, 142.10, -46.70, 149.40,
     &  -47.90, 156.80, -49.30, 164.30, -50.90, 172.00, -52.60, 179.80,
     &  -54.60, 187.70, -56.60, 195.90, -58.70, 204.30, -60.90, 212.90,
     &  -63.30, 222.10, -65.80, 231.90, -68.30, 243.00, -70.80, 256.30/
!  column   89 data, 36 LAT,LNG pairs
	DATA (CGMC(I, 89),I=1,72)/
     &  -71.00, 274.90, -72.00, 294.60, -71.50, 315.40, -69.40, 334.00,
     &  -66.40, 349.10, -63.10,   1.40, -59.90,  12.00, -57.10,  21.70,
     &  -54.70,  31.10, -52.60,  40.50, -50.60,  49.70, -48.80,  58.60,
     &  -47.00,  67.00, -45.50,  75.10, -44.30,  82.90, -43.30,  90.50,
     &  -42.60,  97.80, -42.30, 105.20, -42.20, 112.60, -42.30, 120.00,
     &  -42.60, 127.50, -43.20, 135.00, -43.90, 142.50, -44.90, 150.00,
     &  -46.10, 157.60, -47.60, 165.30, -49.20, 173.10, -50.90, 181.10,
     &  -52.90, 189.20, -54.90, 197.60, -57.00, 206.20, -59.20, 215.10,
     &  -61.50, 224.30, -64.00, 234.20, -66.50, 245.40, -69.00, 258.60/
!  column   90 data, 36 LAT,LNG pairs
	DATA (CGMC(I, 90),I=1,72)/
     &  -69.10, 276.50, -70.00, 295.80, -69.30, 315.60, -66.90, 333.40,
     &  -63.70, 347.80, -60.30, 359.50, -57.10,   9.90, -54.30,  19.40,
     &  -52.00,  28.70, -50.00,  38.20, -48.30,  47.50, -46.60,  56.70,
     &  -45.00,  65.50, -43.50,  73.80, -42.30,  81.80, -41.40,  89.60,
     &  -40.80,  97.10, -40.50, 104.70, -40.40, 112.30, -40.50, 119.90,
     &  -40.80, 127.50, -41.40, 135.20, -42.10, 142.90, -43.10, 150.60,
     &  -44.40, 158.30, -45.80, 166.20, -47.40, 174.20, -49.20, 182.30,
     &  -51.20, 190.70, -53.20, 199.10, -55.30, 207.90, -57.50, 216.90,
     &  -59.80, 226.20, -62.20, 236.20, -64.70, 247.40, -67.10, 260.50/
!  column   91 data, 36 LAT,LNG pairs
	DATA (CGMC(I, 91),I=1,72)/
     &  -67.20, 277.90, -68.00, 296.50, -67.00, 315.80, -64.50, 332.90,
     &  -61.00, 346.70, -57.40, 358.10, -54.10,   8.00, -51.30,  17.30,
     &  -49.10,  26.50, -47.30,  35.90, -45.70,  45.40, -44.30,  54.90,
     &  -42.80,  63.90, -41.50,  72.50, -40.30,  80.80, -39.50,  88.70,
     &  -38.90,  96.50, -38.60, 104.20, -38.50, 112.00, -38.70, 119.80,
     &  -39.00, 127.60, -39.60, 135.40, -40.30, 143.30, -41.30, 151.10,
     &  -42.60, 158.90, -44.10, 167.00, -45.70, 175.10, -47.50, 183.50,
     &  -49.40, 192.00, -51.40, 200.70, -53.50, 209.60, -55.70, 218.60,
     &  -57.90, 228.10, -60.30, 238.10, -62.80, 249.20, -65.30, 262.10/
!  column   92 data, 36 LAT,LNG pairs
	DATA (CGMC(I, 92),I=1,72)/
     &  -65.30, 279.10, -66.00, 297.40, -64.80, 316.10, -61.90, 332.50,
     &  -58.20, 345.80, -54.40, 356.70, -51.00,   6.40, -48.30,  15.50,
     &  -46.20,  24.60, -44.50,  33.90, -43.20,  43.50, -41.90,  53.10,
     &  -40.70,  62.50, -39.50,  71.40, -38.40,  79.80, -37.60,  87.90,
     &  -37.00,  95.80, -36.80, 103.80, -36.70, 111.70, -36.80, 119.50,
     &  -37.20, 127.60, -37.80, 135.60, -38.60, 143.60, -39.60, 151.60,
     &  -40.80, 159.60, -42.30, 167.70, -44.00, 176.10, -45.80, 184.60,
     &  -47.70, 193.20, -49.70, 202.00, -51.70, 211.10, -53.80, 220.30,
     &  -56.00, 229.70, -58.40, 239.80, -60.90, 250.80, -63.40, 263.60/
!  column   93 data, 36 LAT,LNG pairs
	DATA (CGMC(I, 93),I=1,72)/
     &  -63.40, 280.30, -63.90, 298.00, -62.50, 316.30, -59.30, 332.20,
     &  -55.30, 345.00, -51.40, 355.50, -47.90,   4.90, -45.10,  13.80,
     &  -43.20,  22.70, -41.70,  32.00, -40.70,  41.70, -39.70,  51.50,
     &  -38.50,  61.10, -37.40,  70.30, -36.40,  78.80, -35.60,  87.10,
     &  -35.10,  95.10, -34.80, 103.20, -34.90, 111.30, -35.10, 119.50,
     &  -35.50, 127.70, -36.00, 135.90, -36.80, 144.00, -37.80, 152.10,
     &  -39.20, 160.30, -40.60, 168.50, -42.20, 177.00, -44.00, 185.60,
     &  -45.90, 194.30, -47.90, 203.30, -49.90, 212.50, -52.00, 221.70,
     &  -54.20, 231.20, -56.50, 241.20, -59.00, 252.30, -61.40, 265.00/
!  column   94 data, 36 LAT,LNG pairs
	DATA (CGMC(I, 94),I=1,72)/
     &  -61.40, 281.20, -61.90, 298.80, -60.20, 316.60, -56.60, 331.90,
     &  -52.40, 344.20, -48.30, 354.40, -44.90,   3.60, -42.10,  12.30,
     &  -40.30,  21.10, -39.00,  30.30, -38.10,  40.10, -37.40,  50.00,
     &  -36.50,  59.70, -35.40,  69.20, -34.50,  77.90, -33.70,  86.40,
     &  -33.40,  94.70, -33.10, 102.90, -33.10, 111.10, -33.30, 119.50,
     &  -33.70, 127.80, -34.30, 136.00, -35.00, 144.30, -36.00, 152.50,
     &  -37.30, 160.80, -38.80, 169.20, -40.40, 177.80, -42.30, 186.50,
     &  -44.20, 195.50, -46.10, 204.60, -48.10, 213.80, -50.10, 223.10,
     &  -52.30, 232.70, -54.50, 242.70, -57.00, 253.60, -59.50, 266.10/
!  column   95 data, 36 LAT,LNG pairs
	DATA (CGMC(I, 95),I=1,72)/
     &  -59.40, 282.00, -59.80, 299.30, -57.80, 316.80, -54.00, 331.60,
     &  -49.50, 343.50, -45.30, 353.40, -41.80,   2.30, -39.10,  10.90,
     &  -37.30,  19.70, -36.10,  28.80, -35.50,  38.50, -35.00,  48.60,
     &  -34.20,  58.50, -33.40,  68.10, -32.50,  77.10, -31.80,  85.70,
     &  -31.50,  94.10, -31.20, 102.50, -31.30, 110.90, -31.50, 119.40,
     &  -31.90, 127.80, -32.50, 136.20, -33.20, 144.60, -34.20, 153.00,
     &  -35.50, 161.30, -37.00, 169.80, -38.70, 178.60, -40.50, 187.40,
     &  -42.40, 196.50, -44.30, 205.70, -46.20, 215.00, -48.20, 224.40,
     &  -50.30, 234.00, -52.60, 243.90, -55.10, 254.70, -57.50, 267.20/
!  column   96 data, 36 LAT,LNG pairs
	DATA (CGMC(I, 96),I=1,72)/
     &  -57.40, 282.80, -57.70, 299.90, -55.40, 316.90, -51.20, 331.40,
     &  -46.60, 342.80, -42.30, 352.40, -38.90,   1.20, -36.20,   9.70,
     &  -34.50,  18.40, -33.50,  27.50, -32.90,  37.10, -32.60,  47.20,
     &  -32.10,  57.30, -31.40,  67.10, -30.60,  76.30, -29.90,  85.10,
     &  -29.50,  93.60, -29.40, 102.10, -29.50, 110.60, -29.80, 119.30,
     &  -30.20, 127.90, -30.70, 136.40, -31.40, 144.80, -32.40, 153.40,
     &  -33.70, 161.90, -35.20, 170.40, -36.90, 179.30, -38.70, 188.30,
     &  -40.60, 197.50, -42.50, 206.70, -44.40, 216.10, -46.30, 225.60,
     &  -48.40, 235.20, -50.60, 245.20, -53.10, 255.90, -55.60, 268.10/
!  column   97 data, 36 LAT,LNG pairs
	DATA (CGMC(I, 97),I=1,72)/
     &  -55.40, 283.50, -55.60, 300.30, -53.00, 317.10, -48.60, 331.00,
     &  -43.70, 342.00, -39.40, 351.40, -35.90,   0.20, -33.50,   8.70,
     &  -31.80,  17.30, -30.90,  26.30, -30.40,  35.80, -30.30,  45.90,
     &  -29.90,  56.20, -29.30,  66.20, -28.60,  75.50, -28.00,  84.40,
     &  -27.60,  93.10, -27.60, 101.70, -27.70, 110.40, -28.00, 119.20,
     &  -28.40, 128.00, -28.90, 136.60, -29.70, 145.20, -30.70, 153.80,
     &  -31.90, 162.40, -33.40, 171.00, -35.20, 179.90, -37.00, 189.10,
     &  -38.80, 198.40, -40.70, 207.70, -42.50, 217.20, -44.40, 226.80,
     &  -46.40, 236.40, -48.60, 246.30, -51.00, 256.90, -53.50, 269.00/
!  column   98 data, 36 LAT,LNG pairs
	DATA (CGMC(I, 98),I=1,72)/
     &  -53.40, 284.20, -53.40, 300.80, -50.50, 317.10, -45.90, 330.60,
     &  -40.90, 341.40, -36.70, 350.60, -33.30, 359.20, -30.90,   7.70,
     &  -29.20,  16.30, -28.30,  25.20, -28.00,  34.70, -28.00,  44.80,
     &  -27.80,  55.20, -27.30,  65.30, -26.60,  74.80, -26.10,  83.90,
     &  -25.80,  92.70, -25.70, 101.40, -25.90, 110.20, -26.20, 119.00,
     &  -26.60, 128.00, -27.10, 136.80, -27.80, 145.50, -28.80, 154.20,
     &  -30.10, 162.90, -31.70, 171.60, -33.40, 180.60, -35.20, 189.80,
     &  -37.00, 199.20, -38.80, 208.70, -40.60, 218.20, -42.40, 227.80,
     &  -44.40, 237.50, -46.50, 247.40, -49.00, 257.90, -51.50, 269.80/
!  column   99 data, 36 LAT,LNG pairs
	DATA (CGMC(I, 99),I=1,72)/
     &  -51.30, 284.80, -51.20, 301.20, -48.10, 317.20, -43.20, 330.20,
     &  -38.10, 340.70, -33.90, 349.80, -30.70, 358.30, -28.30,   6.70,
     &  -26.70,  15.30, -25.90,  24.30, -25.60,  33.70, -25.70,  43.80,
     &  -25.70,  54.30, -25.30,  64.50, -24.70,  74.10, -24.20,  83.30,
     &  -24.00,  92.20, -23.90, 101.10, -24.10, 110.00, -24.50, 119.10,
     &  -24.80, 128.00, -25.30, 137.00, -26.00, 145.80, -27.00, 154.50,
     &  -28.30, 163.30, -29.90, 172.20, -31.70, 181.30, -33.40, 190.50,
     &  -35.20, 200.00, -36.90, 209.60, -38.70, 219.20, -40.50, 228.80,
     &  -42.40, 238.50, -44.50, 248.40, -46.90, 258.80, -49.40, 270.50/
!  column  100 data, 36 LAT,LNG pairs
	DATA (CGMC(I,100),I=1,72)/
     &  -49.20, 285.30, -49.00, 301.50, -45.60, 317.10, -40.60, 329.70,
     &  -35.60, 340.00, -31.40, 348.90, -28.20, 357.40, -25.90,   5.90,
     &  -24.40,  14.50, -23.60,  23.50, -23.30,  32.90, -23.50,  43.00,
     &  -23.60,  53.40, -23.30,  63.70, -22.80,  73.50, -22.30,  82.80,
     &  -22.00,  91.90, -22.00, 100.80, -22.30, 109.80, -22.70, 119.00,
     &  -23.10, 128.20, -23.60, 137.20, -24.20, 146.10, -25.20, 154.90,
     &  -26.50, 163.70, -28.10, 172.60, -29.90, 181.80, -31.70, 191.20,
     &  -33.40, 200.80, -35.10, 210.40, -36.80, 220.10, -38.50, 229.80,
     &  -40.40, 239.50, -42.40, 249.40, -44.80, 259.70, -47.40, 271.30/
!  column  101 data, 36 LAT,LNG pairs
	DATA (CGMC(I,101),I=1,72)/
     &  -47.10, 285.90, -46.80, 301.80, -43.10, 317.00, -38.00, 329.20,
     &  -33.00, 339.10, -29.00, 348.00, -26.00, 356.60, -23.70,   5.20,
     &  -22.20,  13.80, -21.40,  22.80, -21.20,  32.20, -21.30,  42.20,
     &  -21.60,  52.70, -21.40,  63.10, -20.90,  73.00, -20.40,  82.40,
     &  -20.20,  91.50, -20.20, 100.60, -20.50, 109.70, -20.70, 119.00,
     &  -21.30, 128.30, -21.70, 137.40, -22.40, 146.40, -23.40, 155.20,
     &  -24.80, 164.10, -26.40, 173.20, -28.10, 182.40, -29.90, 191.90,
     &  -31.60, 201.50, -33.20, 211.20, -34.90, 220.90, -36.50, 230.70,
     &  -38.30, 240.40, -40.30, 250.20, -42.70, 260.50, -45.20, 272.00/
!  column  102 data, 36 LAT,LNG pairs
	DATA (CGMC(I,102),I=1,72)/
     &  -45.00, 286.30, -44.50, 302.00, -40.80, 316.80, -35.60, 328.60,
     &  -30.60, 338.40, -26.70, 347.20, -23.70, 355.80, -21.60,   4.40,
     &  -20.10,  13.20, -19.30,  22.20, -19.10,  31.60, -19.30,  41.50,
     &  -19.60,  52.00, -19.50,  62.50, -19.00,  72.50, -18.50,  82.00,
     &  -18.30,  91.10, -18.40, 100.30, -18.70, 109.60, -19.10, 119.00,
     &  -19.50, 128.30, -19.90, 137.60, -20.70, 146.60, -21.60, 155.60,
     &  -22.90, 164.50, -24.60, 173.60, -26.40, 183.00, -28.10, 192.50,
     &  -29.80, 202.20, -31.40, 211.90, -33.00, 221.70, -34.60, 231.50,
     &  -36.30, 241.30, -38.30, 251.10, -40.60, 261.30, -43.10, 272.70/
!  column  103 data, 36 LAT,LNG pairs
	DATA (CGMC(I,103),I=1,72)/
     &  -42.90, 286.80, -42.30, 302.10, -38.30, 316.50, -33.10, 328.00,
     &  -28.30, 337.60, -24.50, 346.50, -21.70, 355.10, -19.60,   3.80,
     &  -18.10,  12.60, -17.20,  21.70, -17.00,  31.10, -17.20,  41.00,
     &  -17.60,  51.40, -17.50,  61.90, -17.10,  72.00, -16.70,  81.60,
     &  -16.50,  90.90, -16.50, 100.10, -16.90, 109.50, -17.30, 119.00,
     &  -17.70, 128.40, -18.10, 137.70, -18.80, 146.90, -19.80, 155.90,
     &  -21.20, 164.90, -22.90, 174.10, -24.60, 183.50, -26.40, 193.10,
     &  -28.00, 202.90, -29.50, 212.70, -31.00, 222.40, -32.60, 232.30,
     &  -34.30, 242.10, -36.10, 251.90, -38.40, 262.10, -41.00, 273.40/
!  column  104 data, 36 LAT,LNG pairs
	DATA (CGMC(I,104),I=1,72)/
     &  -40.70, 287.20, -40.00, 302.10, -36.00, 316.10, -30.80, 327.40,
     &  -26.20, 336.80, -22.40, 345.60, -19.70, 354.40, -17.60,   3.20,
     &  -16.20,  12.10, -15.30,  21.30, -15.00,  30.70, -15.20,  40.50,
     &  -15.60,  50.90, -15.60,  61.50, -15.20,  71.60, -14.80,  81.30,
     &  -14.60,  90.60, -14.70,  99.90, -15.10, 109.40, -15.60, 119.00,
     &  -15.90, 128.60, -16.30, 137.90, -16.90, 147.10, -17.90, 156.20,
     &  -19.40, 165.30, -21.10, 174.50, -22.90, 184.00, -24.60, 193.70,
     &  -26.20, 203.50, -27.60, 213.30, -29.10, 223.10, -30.60, 233.00,
     &  -32.20, 242.80, -34.10, 252.70, -36.30, 262.80, -38.80, 274.00/
!  column  105 data, 36 LAT,LNG pairs
	DATA (CGMC(I,105),I=1,72)/
     &  -38.70, 287.50, -37.90, 302.20, -33.70, 315.70, -28.60, 326.70,
     &  -24.00, 336.10, -20.50, 344.90, -17.80, 353.70, -15.80,   2.60,
     &  -14.40,  11.70, -13.50,  20.90, -13.10,  30.30, -13.30,  40.10,
     &  -13.70,  50.50, -13.80,  61.00, -13.40,  71.10, -12.90,  80.60,
     &  -12.70,  90.40, -12.90,  99.80, -13.40, 109.30, -13.80, 119.00,
     &  -14.10, 128.60, -14.50, 138.10, -15.10, 147.40, -16.10, 156.50,
     &  -17.60, 165.60, -19.40, 174.90, -21.20, 184.50, -22.80, 194.30,
     &  -24.30, 204.10, -25.80, 213.90, -27.20, 223.80, -28.70, 233.70,
     &  -30.20, 243.50, -32.00, 253.40, -34.20, 263.50, -36.80, 274.60/
!  column  106 data, 36 LAT,LNG pairs
	DATA (CGMC(I,106),I=1,72)/
     &  -36.50, 287.90, -35.60, 302.20, -31.50, 315.30, -26.50, 325.90,
     &  -22.10, 335.30, -18.70, 344.20, -16.00, 353.10, -14.10,   2.10,
     &   -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,
     &   -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,
     &   -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00, -12.00, 119.00,
     &  -12.40, 128.70, -12.70, 138.30, -13.20, 147.60, -14.30, 156.80,
     &  -15.80, 165.90, -17.60, 175.30, -19.40, 184.90, -21.00, 194.70,
     &  -22.50, 204.60, -23.80, 214.50, -25.20, 224.40, -26.60, 234.30,
     &  -28.00, 244.20, -29.80, 254.00, -32.10, 264.10, -34.60, 275.10/
!  column  107 data, 36 LAT,LNG pairs
	DATA (CGMC(I,107),I=1,72)/
     &  -34.40, 288.10, -33.40, 302.10, -29.30, 314.80, -24.30, 325.20,
     &  -20.10, 334.60, -16.80, 343.50, -14.40, 352.60, -12.70,   1.70,
     &   -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,
     &   -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,
     &   -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,
     &   -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00, -12.40, 157.00,
     &  -14.00, 166.30, -15.80, 175.70, -17.60, 185.40, -19.30, 195.20,
     &  -20.70, 205.20, -22.00, 215.10, -23.30, 225.00, -24.60, 234.90,
     &  -26.00, 244.80, -27.80, 254.70, -30.00, 264.70, -32.50, 275.50/
!  column  108 data, 36 LAT,LNG pairs
	DATA (CGMC(I,108),I=1,72)/
     &  -32.30, 288.40, -31.20, 302.00, -27.10, 314.20, -22.30, 324.60,
     &  -18.30, 333.90, -15.20, 342.90,  -1.00,  -1.00,  -1.00,  -1.00,
     &   -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,
     &   -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,
     &   -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,
     &   -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,
     &   -1.00,  -1.00, -14.00, 176.10, -15.90, 185.80, -17.50, 195.70,
     &  -18.80, 205.70, -20.00, 215.60, -21.30, 225.50, -22.60, 235.50,
     &  -24.00, 245.40, -25.70, 255.30, -27.80, 265.30, -30.40, 276.00/
!  column  109 data, 36 LAT,LNG pairs
	DATA (CGMC(I,109),I=1,72)/
     &  -30.20, 288.60, -29.10, 301.80, -25.00, 313.70, -20.40, 323.90,
     &  -16.60, 333.20,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,
     &   -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,
     &   -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,
     &   -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,
     &   -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,
     &   -1.00,  -1.00, -12.30, 176.50, -14.10, 186.20, -15.70, 196.10,
     &  -17.00, 206.10, -18.20, 216.10, -19.40, 226.00, -20.70, 236.00,
     &  -22.00, 245.90, -23.60, 255.80, -25.70, 265.90, -28.20, 276.50/
!  column  110 data, 36 LAT,LNG pairs
	DATA (CGMC(I,110),I=1,72)/
     &  -28.00, 288.70, -26.90, 301.60, -23.00, 313.10, -18.50, 323.20,
     &   -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,
     &   -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,
     &   -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,
     &   -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,
     &   -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,
     &   -1.00,  -1.00,  -1.00,  -1.00, -12.30, 186.60, -13.80, 196.50,
     &  -15.10, 206.50, -16.30, 216.50, -17.50, 226.40, -18.70, 236.50,
     &  -20.00, 246.40, -21.60, 256.30, -23.70, 266.30, -26.20, 276.90/
!  column  111 data, 36 LAT,LNG pairs
	DATA (CGMC(I,111),I=1,72)/
     &  -26.00, 288.90, -24.90, 301.30, -20.90, 312.60, -16.80, 322.50,
     &   -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,
     &   -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,
     &   -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,
     &   -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,
     &   -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,
     &   -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00, -12.10, 197.00,
     &  -13.30, 207.00, -14.40, 216.90, -15.50, 226.90, -16.70, 236.90,
     &  -18.00, 246.80, -19.50, 256.80, -21.60, 266.80, -24.10, 277.30/
!  column  112 data, 36 LAT,LNG pairs
	DATA (CGMC(I,112),I=1,72)/
     &  -23.90, 289.10, -22.80, 301.10, -18.90, 312.00,  -1.00,  -1.00,
     &   -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,
     &   -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,
     &   -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,
     &   -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,
     &   -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,
     &   -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,
     &   -1.00,  -1.00, -12.50, 217.40, -13.60, 227.30, -14.80, 237.30,
     &  -16.00, 247.30, -17.60, 257.20, -19.60, 267.20, -22.10, 277.60/
!  column  113 data, 36 LAT,LNG pairs
	DATA (CGMC(I,113),I=1,72)/
     &  -21.80, 289.10, -20.80, 300.90, -17.00, 311.50,  -1.00,  -1.00,
     &   -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,
     &   -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,
     &   -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,
     &   -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,
     &   -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,
     &   -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,
     &   -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00, -12.90, 237.70,
     &  -14.10, 247.40, -15.60, 257.50, -17.50, 267.70, -20.00, 277.90/
!  column  114 data, 36 LAT,LNG pairs
	DATA (CGMC(I,114),I=1,72)/
     &  -19.90, 289.20, -18.80, 300.60,  -1.00,  -1.00,  -1.00,  -1.00,
     &   -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,
     &   -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,
     &   -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,
     &   -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,
     &   -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,
     &   -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,
     &   -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,
     &  -12.20, 248.00, -13.60, 258.00, -15.50, 268.00, -18.00, 278.20/
!  column  115 data, 36 LAT,LNG pairs
	DATA (CGMC(I,115),I=1,72)/
     &  -17.90, 289.20, -16.80, 300.30,  -1.00,  -1.00,  -1.00,  -1.00,
     &   -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,
     &   -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,
     &   -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,
     &   -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,
     &   -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,
     &   -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,
     &   -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,
     &   -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00, -15.90, 278.50/
!  column  116 data, 36 LAT,LNG pairs
	DATA (CGMC(I,116),I=1,72)/
     &   -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,
     &   -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,
     &   27.30,   6.70,  26.10,  18.10,  24.40,  28.80,  -1.00,  -1.00,
     &   -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,
     &   -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,
     &   -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,
     &   -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,
     &   -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,
     &   -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00/
!  column  117 data, 36 LAT,LNG pairs
	DATA (CGMC(I,117),I=1,72)/
     &   -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,
     &   -1.00,  -1.00,  -1.00,  -1.00,  26.80, 342.30,  28.40, 354.60,
     &   28.30,   6.60,  27.40,  18.00,  25.90,  28.70,  24.30,  38.90,
     &   -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,
     &   -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,
     &   -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,
     &   -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,
     &   -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,
     &   -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00/
!  column  118 data, 36 LAT,LNG pairs
	DATA (CGMC(I,118),I=1,72)/
     &   -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,
     &   -1.00,  -1.00,  24.10, 330.50,  27.80, 342.00,  29.50, 354.30,
     &   29.60,   6.50,  28.60,  17.90,  27.50,  28.50,  25.90,  38.90,
     &   24.80,  48.70,  24.30,  58.40,  24.50,  68.20,  25.00,  78.30,
     &   25.30,  88.60,  25.40,  98.90,  25.30, 109.00,  25.30, 119.10,
     &   25.40, 129.30,  25.70, 139.70,  25.60, 150.50,  24.70, 161.60,
     &   -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,
     &   -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,
     &   -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00/
!  column  119 data, 36 LAT,LNG pairs
	DATA (CGMC(I,119),I=1,72)/
     &   -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,
     &   -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  30.60, 354.00,
     &   30.80,   6.20,  30.00,  17.70,  28.80,  28.60,  27.50,  38.90,
     &   26.60,  48.70,  26.20,  58.40,  26.30,  68.20,  26.80,  78.30,
     &   27.10,  88.50,  27.20,  98.80,  27.20, 108.90,  27.20, 119.00,
     &   27.40, 129.20,  27.60, 139.60,  27.60, 150.60,  26.70, 161.70,
     &   25.00, 172.60,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,
     &   -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,
     &   -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00/
!  column  120 data, 36 LAT,LNG pairs
	DATA (CGMC(I,120),I=1,72)/
     &   -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,
     &   -1.00,  -1.00,  26.50, 330.00,  30.10, 341.60,  31.80, 353.90,
     &   32.10,   6.10,  31.50,  17.50,  30.40,  28.50,  29.20,  38.80,
     &   28.30,  48.70,  28.00,  58.40,  28.10,  68.20,  28.60,  78.30,
     &   28.90,  88.50,  29.10,  98.70,  29.10, 108.80,  29.10, 118.90,
     &   29.30, 129.10,  29.60, 139.60,  29.60, 150.60,  28.80, 161.90,
     &   27.10, 172.90,  24.90, 183.40,  -1.00,  -1.00,  -1.00,  -1.00,
     &   -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,
     &   -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00/
!  column  121 data, 36 LAT,LNG pairs
	DATA (CGMC(I,121),I=1,72)/
     &   -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,
     &   -1.00,  -1.00,  27.80, 329.70,  31.30, 341.40,  33.00, 353.70,
     &   33.40,   5.80,  32.90,  17.40,  31.90,  28.40,  30.90,  38.70,
     &   30.10,  48.60,  29.80,  58.40,  30.00,  68.20,  30.40,  78.30,
     &   30.70,  88.50,  30.80,  98.70,  30.90, 108.80,  30.90, 118.90,
     &   31.20, 129.00,  31.60, 139.50,  31.60, 150.60,  30.80, 162.00,
     &   29.20, 173.20,  27.10, 183.80,  25.10, 194.00,  -1.00,  -1.00,
     &   -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,
     &   -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00/
!  column  122 data, 36 LAT,LNG pairs
	DATA (CGMC(I,122),I=1,72)/
     &   -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,
     &   -1.00,  -1.00,  -1.00,  -1.00,  32.50, 341.00,  34.20, 353.30,
     &   34.70,   5.60,  34.40,  17.20,  33.60,  28.20,  32.70,  38.60,
     &   31.90,  48.60,  31.70,  58.40,  31.90,  68.20,  32.20,  78.20,
     &   32.50,  88.40,  32.60,  98.60,  32.70, 108.80,  32.90, 118.80,
     &   33.20, 128.90,  33.50, 139.50,  33.60, 150.50,  32.90, 161.80,
     &   31.30, 173.40,  29.30, 184.20,  27.30, 194.50,  25.50, 204.40,
     &   24.10, 214.20,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,
     &   -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00/
!  column  123 data, 36 LAT,LNG pairs
	DATA (CGMC(I,123),I=1,72)/
     &   -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,
     &   -1.00,  -1.00,  -1.00,  -1.00,  33.80, 340.80,  35.50, 353.00,
     &   36.20,   5.30,  35.90,  17.00,  35.20,  28.00,  34.40,  38.50,
     &   33.80,  48.50,  33.60,  58.30,  33.70,  68.20,  34.00,  78.20,
     &   34.30,  88.40,  34.50,  98.50,  34.60, 108.70,  34.70, 118.70,
     &   35.10, 128.80,  35.50, 139.40,  35.60, 150.60,  35.00, 162.00,
     &   33.50, 173.70,  31.50, 184.60,  29.50, 194.90,  27.70, 204.90,
     &   26.10, 214.70,  24.70, 224.50,  -1.00,  -1.00,  -1.00,  -1.00,
     &   -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00/
!  column  124 data, 36 LAT,LNG pairs
	DATA (CGMC(I,124),I=1,72)/
     &   -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,
     &   27.90, 319.00,  31.90, 329.00,  35.10, 340.60,  36.90, 352.80,
     &   37.60,   5.00,  37.40,  16.80,  36.80,  27.90,  36.10,  38.40,
     &   35.60,  48.50,  35.40,  58.30,  35.60,  68.20,  35.90,  78.20,
     &   36.10,  88.40,  36.30,  98.50,  36.40, 108.60,  36.60, 118.60,
     &   37.00, 128.70,  37.40, 139.40,  37.60, 150.60,  37.00, 162.30,
     &   35.60, 174.00,  33.70, 185.10,  31.60, 195.50,  29.80, 205.50,
     &   28.10, 215.30,  26.60, 225.00,  25.20, 234.70,  -1.00,  -1.00,
     &   -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00/
!  column  125 data, 36 LAT,LNG pairs
	DATA (CGMC(I,125),I=1,72)/
     &   -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  25.50, 310.00,
     &   29.40, 318.70,  33.30, 328.70,  36.40, 340.20,  38.20, 352.30,
     &   39.00,   4.70,  39.00,  15.50,  38.60,  27.60,  37.90,  38.30,
     &   37.50,  48.40,  37.30,  58.30,  37.40,  68.20,  37.70,  78.20,
     &   38.00,  88.40,  38.20,  98.50,  38.30, 108.50,  38.50, 118.50,
     &   39.00, 128.80,  39.40, 139.30,  39.60, 150.60,  39.00, 162.50,
     &   37.70, 174.30,  35.80, 185.50,  33.80, 196.00,  31.90, 206.10,
     &   30.20, 215.90,  28.60, 225.60,  27.10, 235.20,  25.60, 244.70,
     &   -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00,  -1.00/
!  column  126 data, 36 LAT,LNG pairs
	DATA (CGMC(I,126),I=1,72)/
     &   -1.00,  -1.00,  -1.00,  -1.00,  24.50, 302.00,  27.40, 309.70,
     &   31.10, 318.40,  34.70, 328.40,  37.70, 339.80,  39.60, 352.00,
     &   40.50,   4.30,  40.60,  16.20,  40.20,  27.50,  39.70,  38.10,
     &   39.30,  48.30,  39.20,  58.30,  39.30,  68.20,  39.60,  78.20,
     &   39.80,  88.30,  40.00,  98.40,  40.20, 108.40,  40.50, 118.40,
     &   40.90, 128.80,  41.30, 139.30,  41.50, 150.70,  41.10, 162.60,
     &   39.80, 174.60,  38.00, 186.00,  35.90, 196.60,  34.00, 206.70,
     &   32.20, 216.60,  30.50, 226.20,  29.00, 235.70,  27.40, 245.10,
     &   25.80, 254.20,  24.20, 263.00,  -1.00,  -1.00,  -1.00,  -1.00/
!  column  127 data, 36 LAT,LNG pairs
	DATA (CGMC(I,127),I=1,72)/
     &   -1.00,  -1.00,  24.60, 294.30,  26.40, 301.70,  29.20, 309.50,
     &   32.70, 318.20,  36.20, 328.20,  39.10, 339.50,  41.00, 351.60,
     &   42.00,   3.90,  42.20,  15.80,  41.90,  27.10,  41.50,  37.90,
     &   41.20,  48.20,  41.10,  58.20,  41.20,  68.20,  41.40,  78.20,
     &   41.70,  88.30,  41.90,  98.30,  42.10, 108.30,  42.40, 118.30,
     &   42.80, 128.50,  43.30, 139.20,  43.50, 150.70,  43.10, 162.80,
     &   41.90, 174.90,  40.10, 186.40,  38.00, 197.20,  36.10, 207.40,
     &   34.20, 217.20,  32.50, 226.80,  30.80, 236.20,  29.20, 245.50,
     &   27.60, 254.50,  26.10, 263.10,  24.80, 271.30,  -1.00,  -1.00/
!  column  128 data, 36 LAT,LNG pairs
	DATA (CGMC(I,128),I=1,72)/
     &   25.80, 286.80,  26.70, 294.10,  28.40, 301.50,  31.10, 309.20,
     &   34.40, 317.90,  37.70, 327.80,  40.50, 339.10,  42.50, 351.10,
     &   43.50,   3.50,  43.80,  15.40,  43.70,  26.80,  43.40,  37.70,
     &   43.10,  48.00,  43.00,  58.10,  43.10,  68.20,  43.30,  78.20,
     &   43.50,  88.30,  43.80,  98.20,  44.00, 108.20,  44.30, 118.20,
     &   44.70, 128.40,  45.20, 139.30,  45.40, 150.80,  45.00, 163.00,
     &   43.90, 175.20,  42.10, 186.90,  40.10, 197.80,  38.10, 208.10,
     &   36.20, 217.90,  34.40, 227.40,  32.70, 236.80,  31.00, 245.90,
     &   29.40, 254.80,  27.90, 263.30,  26.70, 271.50,  25.90, 279.30/
!  column  129 data, 36 LAT,LNG pairs
	DATA (CGMC(I,129),I=1,72)/
     &   27.80, 286.60,  28.70, 293.80,  30.40, 301.10,  33.00, 308.90,
     &   36.10, 317.70,  39.30, 327.60,  42.00, 338.60,  43.90, 350.60,
     &   45.00,   3.00,  45.40,  14.90,  45.40,  26.50,  45.20,  37.40,
     &   45.00,  47.80,  45.00,  58.00,  45.10,  68.10,  45.20,  78.20,
     &   45.40,  88.20,  45.60,  98.20,  45.90, 108.10,  46.20, 118.20,
     &   46.70, 128.40,  47.20, 139.30,  47.40, 150.90,  47.00, 163.30,
     &   45.90, 175.60,  44.20, 187.40,  42.20, 198.50,  40.10, 208.80,
     &   38.20, 218.60,  36.30, 228.10,  34.50, 237.30,  32.80, 246.30,
     &   31.20, 255.10,  29.80, 263.50,  28.60, 271.60,  27.80, 279.20/
!  column  130 data, 36 LAT,LNG pairs
	DATA (CGMC(I,130),I=1,72)/
     &   29.80, 286.40,  30.70, 293.60,  32.40, 300.90,  34.80, 308.70,
     &   37.70, 317.30,  40.80, 327.20,  43.50, 338.20,  45.40, 350.20,
     &   46.60,   2.40,  47.10,  14.50,  47.10,  26.00,  47.10,  37.10,
     &   46.90,  47.70,  46.90,  58.00,  47.00,  68.00,  47.10,  78.10,
     &   47.30,  88.20,  47.50,  98.10,  47.80, 108.10,  48.10, 118.10,
     &   48.60, 128.50,  49.10, 139.40,  49.30, 151.10,  48.90, 163.60,
     &   47.90, 176.10,  46.20, 188.00,  44.20, 199.10,  42.10, 209.50,
     &   40.10, 219.30,  38.10, 228.80,  36.40, 237.90,  34.70, 246.80,
     &   33.10, 255.40,  31.60, 263.70,  30.50, 271.50,  29.80, 279.10/
!  column  131 data, 36 LAT,LNG pairs
	DATA (CGMC(I,131),I=1,72)/
     &   31.90, 286.20,  32.70, 293.30,  34.30, 300.60,  36.60, 308.40,
     &   39.50, 317.00,  42.40, 326.80,  45.00, 337.70,  46.90, 349.60,
     &   48.10,   1.80,  48.80,  13.80,  49.00,  25.40,  48.90,  36.70,
     &   48.80,  47.50,  48.80,  57.80,  48.90,  68.00,  49.00,  78.10,
     &   49.20,  88.10,  49.40,  98.10,  49.70, 108.00,  50.10, 118.10,
     &   50.60, 128.50,  51.00, 139.60,  51.20, 151.40,  50.90, 163.90,
     &   49.80, 176.50,  48.30, 188.20,  46.10, 199.80,  44.10, 210.20,
     &   42.00, 220.00,  40.00, 229.40,  38.20, 238.50,  36.50, 247.20,
     &   34.90, 255.70,  33.50, 263.80,  32.40, 271.60,  31.80, 279.00/
!  column  132 data, 36 LAT,LNG pairs
	DATA (CGMC(I,132),I=1,72)/
     &   34.00, 286.00,  34.80, 293.10,  36.30, 300.40,  38.50, 308.10,
     &   41.20, 316.70,  44.00, 326.60,  46.50, 337.40,  48.40, 349.10,
     &   49.70,   1.20,  50.40,  13.30,  50.70,  25.10,  50.80,  36.40,
     &   50.80,  47.20,  50.80,  57.60,  50.80,  67.90,  51.00,  78.00,
     &   51.20,  88.00,  51.40,  98.00,  51.70, 108.00,  52.00, 118.10,
     &   52.50, 128.60,  53.00, 139.90,  53.20, 151.70,  52.80, 164.40,
     &   51.80, 177.10,  50.10, 189.20,  48.10, 200.50,  46.00, 211.00,
     &   43.90, 220.80,  41.90, 230.10,  40.00, 239.10,  38.30, 247.70,
     &   36.70, 256.00,  35.40, 263.90,  34.40, 271.60,  33.80, 278.90/
!  column  133 data, 36 LAT,LNG pairs
	DATA (CGMC(I,133),I=1,72)/
     &   35.90, 285.80,  36.80, 292.90,  38.20, 300.00,  40.30, 307.80,
     &   42.90, 316.30,  45.60, 325.90,  48.00, 336.70,  50.00, 348.30,
     &   51.40,   0.50,  52.10,  12.60,  52.50,  24.50,  52.60,  35.90,
     &   52.70,  46.90,  52.70,  57.40,  52.80,  67.80,  52.90,  77.90,
     &   53.10,  88.00,  53.30,  98.00,  53.60, 108.00,  54.00, 118.20,
     &   54.50, 128.80,  54.90, 140.00,  55.00, 152.10,  54.70, 164.90,
     &   53.70, 177.70,  52.10, 189.90,  50.10, 201.30,  47.90, 211.80,
     &   45.80, 221.50,  43.70, 230.80,  41.90, 239.60,  40.20, 248.10,
     &   38.60, 256.30,  37.30, 264.10,  36.30, 271.60,  35.80, 278.80/
!  column  134 data, 36 LAT,LNG pairs
	DATA (CGMC(I,134),I=1,72)/
     &   38.00, 285.60,  38.80, 292.60,  40.20, 299.80,  42.50, 308.00,
     &   44.70, 316.10,  47.30, 325.50,  49.60, 336.10,  51.60, 347.70,
     &   53.00, 359.70,  53.80,  11.90,  54.30,  23.80,  54.60,  35.30,
     &   54.60,  46.40,  54.70,  57.10,  54.80,  67.50,  54.90,  77.80,
     &   55.10,  87.90,  55.30,  97.90,  55.60, 108.00,  55.90, 118.20,
     &   56.40, 129.00,  56.80, 140.30,  57.00, 152.60,  56.60, 165.40,
     &   55.60, 178.40,  54.00, 190.70,  52.00, 202.10,  49.80, 212.60,
     &   47.70, 222.30,  45.60, 231.50,  43.70, 240.20,  42.00, 248.50,
     &   40.40, 256.50,  39.20, 264.20,  38.30, 271.60,  37.80, 278.70/
!  column  135 data, 36 LAT,LNG pairs
	DATA (CGMC(I,135),I=1,72)/
     &   40.00, 285.50,  40.80, 292.40,  42.20, 299.50,  44.10, 307.20,
     &   46.40, 315.60,  48.90, 325.00,  51.20, 335.50,  53.10, 346.90,
     &   54.60, 358.90,  55.60,  11.10,  56.10,  23.10,  56.40,  34.70,
     &   56.60,  46.00,  56.70,  56.80,  56.80,  67.30,  56.90,  77.60,
     &   57.10,  87.80,  57.30,  97.90,  57.50, 108.00,  57.90, 118.30,
     &   58.40, 129.20,  58.80, 140.80,  58.90, 153.10,  58.50, 166.10,
     &   57.50, 179.10,  55.90, 191.60,  53.80, 203.00,  51.70, 213.50,
     &   49.50, 223.20,  47.40, 232.20,  45.50, 240.80,  43.80, 249.00,
     &   42.30, 256.80,  41.10, 264.40,  40.20, 271.60,  39.80, 278.60/
!  column  136 data, 36 LAT,LNG pairs
	DATA (CGMC(I,136),I=1,72)/
     &   42.10, 285.30,  42.80, 292.20,  44.10, 299.20,  46.00, 306.80,
     &   48.20, 315.20,  50.60, 324.40,  52.80, 334.80,  54.70, 346.10,
     &   56.30, 358.00,  57.30,  10.20,  57.90,  22.30,  58.30,  34.00,
     &   58.60,  45.40,  58.70,  56.40,  58.80,  67.00,  58.90,  77.40,
     &   59.10,  87.70,  59.30,  97.90,  59.50, 108.10,  59.90, 118.50,
     &   60.40, 129.50,  60.70, 141.30,  60.80, 153.80,  60.40, 166.90,
     &   59.30, 180.00,  57.70, 192.50,  55.70, 203.90,  53.50, 214.40,
     &   51.30, 224.00,  49.20, 233.00,  47.40, 241.50,  45.60, 249.50,
     &   44.20, 257.20,  43.00, 264.50,  42.30, 271.60,  41.90, 278.50/
!  column  137 data, 36 LAT,LNG pairs
	DATA (CGMC(I,137),I=1,72)/
     &   44.10, 285.20,  44.80, 291.90,  46.10, 298.90,  47.90, 306.40,
     &   50.00, 314.70,  52.20, 323.80,  54.40, 334.00,  56.40, 345.20,
     &   57.90, 356.90,  59.00,   9.20,  59.80,  21.30,  60.20,  33.20,
     &   60.50,  44.70,  60.70,  55.90,  60.80,  66.70,  60.90,  77.20,
     &   61.10,  87.50,  61.30,  97.80,  61.60, 108.10,  62.00, 118.80,
     &   62.30, 130.00,  62.60, 141.80,  62.60, 154.60,  62.10, 167.80,
     &   61.10, 181.00,  59.50, 193.50,  57.50, 205.00,  55.30, 215.40,
     &   53.20, 224.90,  51.10, 233.80,  49.20, 242.10,  47.50, 250.00,
     &   46.10, 257.50,  45.00, 264.70,  44.30, 271.70,  43.90, 278.40/
!  column  138 data, 36 LAT,LNG pairs
	DATA (CGMC(I,138),I=1,72)/
     &   46.10, 284.90,  46.90, 291.60,  48.10, 298.60,  49.80, 306.00,
     &   51.70, 314.20,  53.90, 323.20,  56.10, 333.20,  58.00, 344.10,
     &   59.60, 355.80,  60.80,   8.00,  61.60,  20.20,  62.20,  32.20,
     &   62.50,  43.90,  62.70,  55.20,  62.80,  66.20,  62.90,  76.90,
     &   63.10,  87.30,  63.40,  97.70,  63.60, 108.20,  64.00, 119.10,
     &   64.30, 130.40,  64.50, 142.60,  64.50, 155.50,  64.00, 168.90,
     &   62.90, 182.20,  61.20, 194.70,  59.30, 206.10,  57.10, 216.50,
     &   55.00, 225.90,  52.90, 234.60,  51.00, 242.70,  49.40, 250.50,
     &   48.00, 257.80,  46.90, 264.80,  46.30, 271.70,  45.90, 278.40/
!  column  139 data, 36 LAT,LNG pairs
	DATA (CGMC(I,139),I=1,72)/
     &   48.20, 284.80,  48.90, 291.40,  50.10, 298.30,  51.70, 305.60,
     &   53.50, 313.60,  55.60, 322.50,  57.70, 332.30,  59.70, 343.10,
     &   61.30, 354.60,  62.60,   6.80,  63.50,  18.90,  64.10,  31.10,
     &   64.50,  42.90,  64.70,  54.40,  64.90,  65.60,  65.00,  76.50,
     &   65.20,  87.10,  65.40,  97.70,  65.70, 108.40,  66.00, 119.40,
     &   66.30, 131.10,  66.50, 143.60,  66.40, 156.60,  65.80, 170.10,
     &   64.70, 183.50,  63.00, 196.00,  61.00, 207.40,  58.90, 217.60,
     &   56.70, 226.80,  54.70, 235.40,  52.80, 243.40,  51.20, 251.00,
     &   49.90, 258.20,  48.90, 265.00,  48.30, 271.70,  48.00, 278.30/
!  column  140 data, 36 LAT,LNG pairs
	DATA (CGMC(I,140),I=1,72)/
     &   50.20, 284.60,  50.90, 291.10,  52.00, 297.90,  53.60, 305.10,
     &   55.40, 313.00,  57.40, 321.60,  59.40, 331.30,  61.40, 341.80,
     &   63.00, 353.20,  64.30,   5.20,  65.30,  17.50,  66.00,  29.80,
     &   66.50,  41.70,  66.80,  53.50,  66.90,  64.90,  67.10,  75.90,
     &   67.30,  86.80,  67.50,  97.70,  67.80, 108.60,  68.10, 120.00,
     &   68.30, 132.00,  68.40, 144.60,  68.20, 158.00,  67.60, 171.70,
     &   66.40, 185.10,  64.70, 197.40,  62.70, 208.70,  60.60, 218.80,
     &   58.50, 227.90,  56.50, 236.30,  54.70, 244.10,  53.20, 251.50,
     &   51.80, 258.50,  50.90, 265.30,  50.30, 271.80,  50.00, 278.20/
!  column  141 data, 36 LAT,LNG pairs
	DATA (CGMC(I,141),I=1,72)/
     &   52.30, 284.50,  53.00, 290.90,  54.10, 297.50,  55.50, 304.60,
     &   57.20, 312.30,  59.20, 320.70,  61.10, 330.10,  63.00, 340.50,
     &   64.70, 351.60,  66.10,   3.40,  67.10,  15.70,  67.90,  28.00,
     &   68.40,  40.30,  68.80,  52.30,  69.00,  64.00,  69.20,  75.30,
     &   69.40,  86.50,  69.60,  97.60,  69.90, 108.90,  70.10, 120.60,
     &   70.30, 132.90,  70.40, 146.00,  70.10, 159.70,  69.30, 173.40,
     &   68.10, 186.80,  66.40, 199.20,  64.40, 210.30,  62.40, 220.20,
     &   60.30, 229.10,  58.40, 237.30,  56.60, 244.90,  55.10, 252.10,
     &   53.80, 258.90,  52.80, 265.50,  52.30, 271.90,  52.10, 278.20/
!  column  142 data, 36 LAT,LNG pairs
	DATA (CGMC(I,142),I=1,72)/
     &   54.40, 284.30,  55.00, 290.60,  56.10, 297.10,  57.40, 304.00,
     &   59.10, 311.60,  60.90, 319.80,  62.80, 328.90,  64.70, 338.90,
     &   66.40, 349.80,  67.80,   1.50,  68.90,  13.70,  69.80,  26.10,
     &   70.40,  38.50,  70.90,  50.80,  71.10,  62.80,  71.40,  74.50,
     &   71.60,  86.00,  71.80,  97.60,  72.00, 109.30,  72.20, 121.50,
     &   72.40, 134.20,  72.30, 147.70,  71.90, 161.70,  71.00, 175.60,
     &   69.70, 188.90,  68.00, 201.00,  66.10, 212.00,  64.00, 221.70,
     &   61.90, 230.80,  60.10, 238.40,  58.40, 245.70,  57.00, 252.70,
     &   55.80, 259.30,  54.90, 265.80,  54.40, 272.00,  54.20, 278.10/
!  column  143 data, 36 LAT,LNG pairs
	DATA (CGMC(I,143),I=1,72)/
     &   56.50, 284.10,  57.10, 290.20,  58.00, 296.60,  59.30, 303.40,
     &   60.90, 310.70,  62.60, 318.70,  64.50, 327.50,  66.30, 337.20,
     &   68.00, 347.80,  69.50, 359.20,  70.70,  11.30,  71.70,  23.80,
     &   72.40,  36.40,  72.90,  49.00,  73.30,  61.40,  73.50,  73.50,
     &   73.80,  85.50,  74.00,  97.60,  74.20, 109.80,  74.30, 122.50,
     &   74.40, 135.90,  74.20, 149.80,  73.70, 164.20,  72.70, 178.10,
     &   71.40, 191.30,  69.70, 203.30,  67.80, 213.90,  65.80, 223.30,
     &   63.80, 231.80,  61.90, 239.40,  60.30, 246.60,  58.90, 253.40,
     &   57.70, 259.80,  56.90, 266.00,  56.40, 272.10,  56.30, 278.10/
!  column  144 data, 36 LAT,LNG pairs
	DATA (CGMC(I,144),I=1,72)/
     &   58.60, 283.90,  59.10, 289.90,  60.00, 296.00,  61.30, 302.60,
     &   62.80, 309.70,  64.40, 317.40,  66.20, 325.90,  68.00, 335.20,
     &   69.70, 345.50,  71.20, 356.50,  72.50,   8.50,  73.50,  20.90,
     &   74.30,  33.70,  74.90,  46.60,  75.40,  59.40,  75.70,  72.10,
     &   76.00,  84.70,  76.20,  97.50,  76.30, 110.50,  76.50, 124.00,
     &   76.40, 138.10,  76.10, 152.70,  75.50, 167.30,  74.40, 181.40,
     &   73.00, 194.30,  71.30, 205.90,  69.30, 216.10,  67.40, 225.10,
     &   65.50, 233.30,  63.70, 240.70,  62.20, 247.60,  60.80, 254.20,
     &   59.70, 260.40,  59.00, 266.40,  58.50, 272.20,  58.40, 278.10/
!  column  145 data, 36 LAT,LNG pairs
	DATA (CGMC(I,145),I=1,72)/
     &   60.70, 283.70,  61.30, 289.40,  62.10, 295.50,  63.30, 301.80,
     &   64.70, 308.50,  66.20, 315.90,  67.90, 324.00,  69.60, 332.90,
     &   71.30, 342.70,  72.80, 353.50,  74.10,   5.00,  75.30,  17.20,
     &   76.20,  30.10,  76.90,  43.40,  77.50,  56.80,  77.90,  70.10,
     &   78.20,  83.70,  78.40,  97.40,  78.60, 111.40,  78.60, 126.00,
     &   78.40, 141.00,  78.00, 156.30,  77.20, 171.30,  76.00, 185.20,
     &   74.50, 197.80,  72.80, 209.00,  70.90, 218.70,  69.10, 227.30,
     &   67.30, 235.10,  65.60, 242.20,  64.10, 248.80,  62.80, 255.00,
     &   61.70, 261.00,  61.00, 266.70,  60.60, 272.40,  60.50, 278.00/
!  column  146 data, 36 LAT,LNG pairs
	DATA (CGMC(I,146),I=1,72)/
     &   62.80, 283.50,  63.30, 289.00,  64.10, 294.70,  65.20, 300.80,
     &   66.50, 307.20,  68.00, 314.20,  69.60, 321.80,  71.20, 330.10,
     &   72.80, 339.30,  74.40, 349.60,  75.80,   0.50,  77.10,  12.50,
     &   78.10,  25.30,  78.90,  38.90,  79.60,  53.00,  80.10,  67.20,
     &   80.50,  82.10,  80.70,  97.30,  80.80, 112.90,  80.70, 129.10,
     &   80.40, 145.40,  79.80, 161.40,  78.80, 176.30,  77.60, 190.10,
     &   76.00, 202.20,  74.30, 212.60,  72.50, 221.80,  70.70, 229.90,
     &   68.90, 237.10,  67.40, 243.80,  66.00, 250.10,  64.80, 256.00,
     &   63.80, 261.70,  63.20, 267.20,  62.70, 272.60,  62.60, 278.00/
!  column  147 data, 36 LAT,LNG pairs
	DATA (CGMC(I,147),I=1,72)/
     &   65.00, 283.20,  65.50, 288.40,  66.20, 293.90,  67.20, 299.60,
     &   68.40, 305.70,  69.80, 312.20,  71.30, 319.20,  72.80, 326.70,
     &   74.30, 335.10,  75.90, 344.90,  77.40, 355.10,  78.70,   6.50,
     &   79.80,  19.00,  80.80,  32.40,  81.60,  47.10,  82.30,  62.70,
     &   82.70,  79.50,  83.00,  97.10,  83.00, 115.30,  82.80, 133.80,
     &   82.40, 152.10,  81.50, 168.70,  80.40, 183.30,  78.90, 196.20,
     &   77.40, 207.40,  75.70, 216.90,  73.90, 225.20,  72.30, 232.70,
     &   70.60, 239.40,  69.10, 245.60,  67.80, 251.50,  66.70, 257.10,
     &   65.90, 262.50,  65.20, 267.70,  64.90, 272.80,  64.80, 278.00/
!  column  148 data, 36 LAT,LNG pairs
	DATA (CGMC(I,148),I=1,72)/
     &   67.20, 282.80,  67.60, 287.80,  68.30, 292.90,  69.10, 298.30,
     &   70.20, 303.90,  71.50, 309.80,  72.90, 316.30,  74.30, 323.10,
     &   75.80, 330.70,  77.30, 339.10,  78.70, 348.50,  80.20, 358.40,
     &   81.40,  10.10,  82.60,  22.80,  83.60,  37.70,  84.40,  54.70,
     &   85.00,  74.60,  85.30,  96.80,  85.30, 120.20,  84.90, 142.80,
     &   84.10, 162.60,  83.00, 178.90,  81.70, 192.80,  80.30, 204.20,
     &   78.70, 213.90,  77.00, 222.20,  75.40, 229.50,  73.80, 236.10,
     &   72.30, 242.20,  70.90, 247.90,  69.70, 253.30,  68.70, 258.40,
     &   67.90, 263.40,  67.40, 268.30,  67.00, 273.10,  67.00, 278.00/
!  column  149 data, 36 LAT,LNG pairs
	DATA (CGMC(I,149),I=1,72)/
     &   69.50, 282.40,  69.80, 287.10,  70.40, 291.70,  71.10, 296.60,
     &   72.10, 301.60,  73.20, 307.00,  74.50, 312.60,  75.80, 318.50,
     &   77.20, 325.10,  78.60, 332.20,  80.00, 339.80,  81.40, 348.00,
     &   82.70, 357.60,  84.00,   8.40,  85.20,  20.90,  86.30,  37.70,
     &   87.20,  61.50,  87.70,  95.80,  87.50, 133.80,  86.60, 161.80,
     &   85.50, 180.90,  84.20, 194.70,  82.80, 205.30,  81.30, 214.10,
     &   79.80, 221.80,  78.20, 228.60,  76.70, 234.70,  75.20, 240.30,
     &   73.90, 245.50,  72.70, 250.50,  71.60, 255.40,  70.70, 260.00,
     &   70.00, 264.60,  69.50, 269.00,  69.20, 273.50,  69.20, 278.00/
!  column  150 data, 36 LAT,LNG pairs
	DATA (CGMC(I,150),I=1,72)/
     &   71.60, 282.10,  71.90, 286.20,  72.40, 290.30,  73.10, 294.60,
     &   73.90, 299.00,  74.90, 303.50,  76.00, 308.20,  77.10, 313.20,
     &   78.30, 318.40,  79.60, 323.70,  80.90, 329.20,  82.20, 335.00,
     &   83.50, 341.00,  84.80, 347.10,  86.10, 353.50,  87.40,   0.00,
     &   88.60,   4.40,  89.90,   8.40,  88.80, 197.70,  87.50, 202.80,
     &   86.10, 209.20,  84.80, 215.30,  83.40, 221.20,  82.00, 226.30,
     &   80.60, 231.10,  79.20, 236.10,  77.90, 240.80,  76.60, 245.20,
     &   75.50, 249.50,  74.40, 253.80,  73.50, 258.00,  72.70, 262.00,
     &   72.20, 266.00,  71.80, 270.00,  71.50, 274.00,  71.50, 278.00/
!  column  151 data, 36 LAT,LNG pairs
	DATA (CGMC(I,151),I=1,72)/
     &   73.80, 281.40,  74.10, 285.00,  74.50, 288.50,  75.00, 292.00,
     &   75.70, 295.60,  76.50, 299.20,  77.40, 302.80,  78.30, 306.50,
     &   79.30, 310.20,  80.40, 313.60,  81.50, 316.60,  82.60, 319.30,
     &   83.70, 321.20,  84.80, 322.40,  85.80, 320.80,  86.70, 314.40,
     &   87.40, 301.20,  87.60, 279.70,  87.50, 256.50,  86.60, 245.60,
     &   85.70, 239.60,  84.60, 238.00,  83.40, 238.80,  82.30, 240.20,
     &   81.20, 242.40,  80.10, 245.00,  79.10, 248.00,  77.90, 251.30,
     &   77.00, 254.50,  76.10, 257.80,  75.40, 261.10,  74.70, 264.40,
     &   74.30, 267.80,  73.90, 271.20,  73.80, 274.60,  73.70, 278.00/
!  column  152 data, 36 LAT,LNG pairs
	DATA (CGMC(I,152),I=1,72)/
     &   76.30, 280.90,  76.20, 283.40,  76.50, 286.10,  76.90, 288.70,
     &   77.60, 291.30,  78.10, 293.80,  78.70, 296.20,  79.40, 298.40,
     &   80.10, 300.30,  80.90, 302.20,  81.70, 303.20,  82.40, 303.60,
     &   83.20, 303.00,  83.90, 301.50,  84.50, 298.40,  84.90, 293.30,
     &   85.20, 286.40,  85.40, 278.50,  85.70, 269.50,  84.80, 264.50,
     &   84.30, 259.60,  83.70, 256.70,  82.90, 255.20,  82.20, 254.30,
     &   81.40, 254.70,  80.60, 255.90,  79.90, 257.40,  79.10, 259.10,
     &   78.40, 260.90,  77.80, 263.00,  77.30, 265.40,  76.80, 267.80,
     &   76.50, 270.30,  76.20, 272.90,  76.00, 275.50,  76.00, 278.10/
!  column  153 data, 36 LAT,LNG pairs
	DATA (CGMC(I,153),I=1,72)/
     &   78.30, 279.70,  78.50, 281.40,  78.60, 282.90,  78.80, 284.40,
     &   79.10, 285.70,  79.40, 287.50,  79.80, 289.00,  80.10, 290.30,
     &   80.50, 291.00,  80.90, 291.30,  81.40, 290.10,  81.80, 289.70,
     &   82.10, 288.80,  82.40, 287.50,  82.70, 285.70,  82.80, 283.50,
     &   82.90, 280.90,  83.00, 278.40,  82.90, 275.70,  82.80, 273.20,
     &   82.50, 271.10,  82.30, 269.60,  82.00, 268.30,  81.60, 267.40,
     &   81.20, 266.90,  80.80, 267.10,  80.40, 267.30,  80.00, 267.90,
     &   79.60, 268.60,  79.40, 269.80,  79.10, 271.00,  78.80, 272.30,
     &   78.60, 273.70,  78.50, 275.10,  78.40, 276.50,  78.40, 278.20/
!
	END
